@@ -3,7 +3,7 @@
  *  Summary:    Functions related to ranged attacks.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:48:37.562004Z $
  *
  *  Change History (most recent first):
  *
@@ -206,10 +206,11 @@ bool nasty_beam( struct monsters *mon, struct bolt &beam );
  * called from: ability - it_use3 - item_use - mstuff2 - religion -
  *              spells - spells4
  * *********************************************************************** */
-void explosion( struct bolt &pbolt, bool hole_in_the_middle = false,
+void explosion( bolt &pbolt, bool hole_in_the_middle = false,
                 bool explode_in_wall = false,
                 bool stop_at_statues = true,
-                bool stop_at_walls   = true);
+                bool stop_at_walls   = true,
+                bool show_more       = true);
 
 // last updated 22jan2001 {gdl}
 /* ***********************************************************************
@@ -223,7 +224,8 @@ int mons_adjust_flavoured( struct monsters *monster, struct bolt &pbolt,
  * called from: ability - item_use - spell
  * returns true if messages were generated during the enchantment
  * *********************************************************************** */
-bool mass_enchantment( enchant_type wh_enchant, int pow, int who );
+bool mass_enchantment( enchant_type wh_enchant, int pow, int who,
+                       int *m_succumbed = NULL, int *m_attempted = NULL );
 
 
 /* ***********************************************************************
