@@ -1,5 +1,5 @@
 /*
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T16:15:56.136594Z $
  */
 
 #include "AppHdr.h"
@@ -1497,10 +1497,14 @@ static int food_do_eat(lua_State *ls)
 
 static int food_prompt_floor(lua_State *ls)
 {
-    bool eaten = false;
-    if (!you.turn_is_over && (eaten = eat_from_floor()))
-        burden_change();
-    lua_pushboolean(ls, eaten);
+    int eaten = 0;
+    if (!you.turn_is_over)
+    {
+        eaten = eat_from_floor();
+        if ( eaten == 1 )
+            burden_change();
+    }
+    lua_pushboolean(ls, (eaten != 0));
     return (1);
 }
 
