@@ -3,7 +3,7 @@
  *  Summary:    Misc stuff.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: zelgadis $ on $Date: 2007-12-09T07:23:27.897226Z $
  *
  *  Change History (most recent first):
  *
@@ -478,7 +478,7 @@ void random_uselessness(unsigned char ru, unsigned char sc_read_2)
 }                               // end random_uselessness()
 
 static int find_acquirement_subtype(object_class_type class_wanted,
-                                    int &unique)
+                                    int &quantity)
 {
     ASSERT(class_wanted != OBJ_RANDOM);
     
@@ -522,7 +522,7 @@ static int find_acquirement_subtype(object_class_type class_wanted,
             // but it's easier to just give them a potion of blood
             // class type is set elsewhere
             type_wanted = POT_BLOOD;
-            unique = 2 + random2(4);
+            quantity = 2 + random2(4);
         }
         else 
         {
@@ -545,13 +545,12 @@ static int find_acquirement_subtype(object_class_type class_wanted,
             }
         }
 
-        // quantity is handled by unique for food
-        unique = 3 + random2(5);
+        quantity = 3 + random2(5);
 
         // giving more of the lower food value items
         if (type_wanted == FOOD_HONEYCOMB || type_wanted == FOOD_CHUNK)
         {
-            unique += random2avg(10, 2);
+            quantity += random2avg(10, 2);
         }
     }
     else if (class_wanted == OBJ_WEAPONS)
@@ -673,13 +672,6 @@ static int find_acquirement_subtype(object_class_type class_wanted,
         // OBJ_RANDOM is body armour and handled below
         type_wanted = (coinflip()) ? OBJ_RANDOM : ARM_SHIELD + random2(5);
 
-        // mutation specific problems (horns allow caps)
-        if ((you.mutation[MUT_HOOVES] && type_wanted == ARM_BOOTS)
-            || (you.mutation[MUT_CLAWS] >= 3 && type_wanted == ARM_GLOVES))
-        {
-            type_wanted = OBJ_RANDOM;
-        }
-
         // some species specific fitting problems
         switch (you.species)
         {
@@ -699,7 +691,9 @@ static int find_acquirement_subtype(object_class_type class_wanted,
         case SP_UNK1_DRACONIAN:
         case SP_BASE_DRACONIAN:
         case SP_SPRIGGAN:
-            if (type_wanted == ARM_GLOVES || type_wanted == ARM_BOOTS)
+            if (type_wanted == ARM_GLOVES || type_wanted == ARM_BOOTS
+                || type_wanted == ARM_CENTAUR_BARDING
+                || type_wanted == ARM_NAGA_BARDING)
             {
                 type_wanted = ARM_ROBE;  // no heavy armour
             }
@@ -721,8 +715,43 @@ static int find_acquirement_subtype(object_class_type class_wanted,
                 type_wanted = OBJ_RANDOM;
             break;
 
-        default:
+        case SP_NAGA:
+            if (type_wanted == ARM_BOOTS || type_wanted == ARM_CENTAUR_BARDING)
+                type_wanted = ARM_NAGA_BARDING;
             break;
+
+        case SP_CENTAUR:
+            if (type_wanted == ARM_BOOTS || type_wanted == ARM_NAGA_BARDING)
+                type_wanted = ARM_CENTAUR_BARDING;
+            break;
+
+        default:
+            if (type_wanted == ARM_CENTAUR_BARDING 
+                || type_wanted == ARM_NAGA_BARDING)
+            {
+                type_wanted = ARM_BOOTS;
+            }
+            break;
+        }
+
+        // mutation specific problems (horns allow caps)
+        if ((you.mutation[MUT_HOOVES] && type_wanted == ARM_BOOTS)
+            || (you.mutation[MUT_CLAWS] >= 3 && type_wanted == ARM_GLOVES))
+        {
+            type_wanted = OBJ_RANDOM;
+        }
+
+        // Do this here, before acquirement()'s call to can_wear_armour(),
+        // so that caps will be just as common as helmets for those
+        // that can't wear helmets.
+        if (type_wanted == ARM_HELMET
+            && ((you.species >= SP_OGRE && you.species <= SP_OGRE_MAGE)
+                || player_genus(GENPC_DRACONIAN)
+                || you.species == SP_KENKU
+                || you.species == SP_SPRIGGAN
+                || you.mutation[MUT_HORNS]))
+        {
+            type_wanted = ARM_CAP;
         }
 
         // Now we'll randomly pick a body armour (light only in the
@@ -1117,7 +1146,7 @@ static int find_acquirement_subtype(object_class_type class_wanted,
 bool acquirement(object_class_type class_wanted, int agent)
 {
     int thing_created = 0;
-    int unique = 1;
+    int quant = 1;
 
     while (class_wanted == OBJ_RANDOM)
     {
@@ -1152,15 +1181,15 @@ bool acquirement(object_class_type class_wanted, int agent)
     {
         for (int item_tries = 0; item_tries < 40; item_tries++)
         {
-            unique = 1;
-            int type_wanted = find_acquirement_subtype(class_wanted, unique);
+            quant = 1;
+            int type_wanted = find_acquirement_subtype(class_wanted, quant);
             
             // clobber class_wanted for vampires
             if (you.species == SP_VAMPIRE && class_wanted == OBJ_FOOD)
                 class_wanted = OBJ_POTIONS;
 
             // BCR - unique is now used for food quantity.
-            thing_created = items( unique, class_wanted, type_wanted, true, 
+            thing_created = items( 1, class_wanted, type_wanted, true, 
                                    MAKE_GOOD_ITEM, 250 );
 
             if (thing_created == NON_ITEM)
@@ -1216,6 +1245,8 @@ bool acquirement(object_class_type class_wanted, int agent)
         // give some more gold
         if ( class_wanted == OBJ_GOLD )
             thing.quantity += 150;
+        else if (quant > 1)
+            thing.quantity = quant;
     
         // remove curse flag from item
         do_uncurse_item( thing );
@@ -1352,66 +1383,6 @@ bool acquirement(object_class_type class_wanted, int agent)
                     thing.plus2 = std::max(static_cast<int>(thing.plus2), 0);
             }
         }
-        else if (thing.base_type == OBJ_ARMOUR
-                 && !is_fixed_artefact( thing )
-                 && !is_unrandom_artefact( thing ))
-        {
-            // HACK: make unwearable hats and boots wearable
-            // Note: messing with fixed artefacts is probably very bad.
-            switch (thing.sub_type)
-            {
-            case ARM_HELMET:
-                if ((get_helmet_type(thing) == THELM_HELM
-                        || get_helmet_type(thing) == THELM_HELMET)
-                    && ((you.species >= SP_OGRE && you.species <= SP_OGRE_MAGE)
-                        || player_genus(GENPC_DRACONIAN)
-                        || you.species == SP_KENKU
-                        || you.species == SP_SPRIGGAN
-                        || you.mutation[MUT_HORNS]))
-                {
-                    // turn it into a cap or wizard hat
-                    set_helmet_type(thing, 
-                                    coinflip() ? THELM_CAP : THELM_WIZARD_HAT);
-
-                    thing.colour = random_colour();
-                }
-                break;
-
-            case ARM_BOOTS:
-                if (you.species == SP_NAGA)
-                    thing.sub_type = ARM_NAGA_BARDING;
-                else if (you.species == SP_CENTAUR)
-                    thing.sub_type = ARM_CENTAUR_BARDING;
-
-                // fix illegal barding ego types caused by above hack
-                if (thing.sub_type != ARM_BOOTS &&
-                    get_armour_ego_type(thing) == SPARM_RUNNING)
-                {
-                    set_item_ego_type( thing, OBJ_ARMOUR, SPARM_NORMAL );
-                }
-                break;
-
-            case ARM_NAGA_BARDING:
-            case ARM_CENTAUR_BARDING:
-                // make barding appropriate
-                if (you.species == SP_NAGA )
-                    thing.sub_type = ARM_NAGA_BARDING;
-                else if ( you.species == SP_CENTAUR )
-                    thing.sub_type = ARM_CENTAUR_BARDING;
-                else
-                {
-                    thing.sub_type = ARM_BOOTS;
-                    // Fix illegal ego types
-                    if (get_armour_ego_type(thing) == SPARM_COLD_RESISTANCE ||
-                        get_armour_ego_type(thing) == SPARM_FIRE_RESISTANCE)
-                        set_item_ego_type(thing, OBJ_ARMOUR, SPARM_NORMAL);
-                }
-                break;
-
-            default:
-                break;
-            }
-        }
 
         move_item_to_grid( &thing_created, you.x_pos, you.y_pos );
 
