@@ -3,7 +3,7 @@
  *  Summary:    Fixed size 2D vector class that asserts if you do something bad.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:05:11.686405Z $
  *
  *  Change History (most recent first):
  *
@@ -168,7 +168,7 @@ public:
 
     virtual bool incapacitated() const
     {
-        return paralysed() || confused() || caught();
+        return paralysed() || asleep() || confused() || caught();
     }
 
     virtual int holy_aura() const
@@ -1399,7 +1399,8 @@ public:
     void reset_options();
 
     void read_option_line(const std::string &s, bool runscripts = false);
-    void read_options(InitLineInput &, bool runscripts);
+    void read_options(InitLineInput &, bool runscripts,
+                      bool clear_aliases = true);
 
 public:
     // View options
