@@ -3,7 +3,7 @@
  *  Summary:    Functions used to save and load levels/games.
  *  Written by: Linley Henzell and Alexey Guzeev
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T14:56:28.564268Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T15:49:30.996463Z $
  *
  *  Change History (most recent first):
  *
@@ -341,7 +341,6 @@ static bool create_dirs(const std::string &dir)
     for (int i = 0, size = segments.size(); i < size; ++i)
     {
         path += segments[i];
-        path += FILE_SEPARATOR;
 
         // Handle absolute paths correctly.
         if (i == 0 && dir.size() && dir[0] == FILE_SEPARATOR)
@@ -349,6 +348,8 @@ static bool create_dirs(const std::string &dir)
 
         if (!dir_exists(path) && create_directory(path.c_str()))
             return (false);
+
+        path += FILE_SEPARATOR;
     }
     return (true);
 }
@@ -1306,10 +1307,7 @@ void restore_game(void)
     char minorVersion = 0;
 
     if (!determine_version(charf, majorVersion, minorVersion))
-    {
-        perror("\nSavefile appears to be invalid.\n");
-        end(-1);
-    }
+        end(-1, false, "\nSavefile appears to be invalid.\n");
 
     restore_version(charf, majorVersion, minorVersion);
 
