@@ -3,7 +3,7 @@
  *  Summary:    Functions for eating and butchering.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T16:15:56.136594Z $
  *
  *  Change History (most recent first):
  *
@@ -88,7 +88,7 @@ bool can_ingest(int what_isit, int kindof_thing, bool suppress_msg,
 
 void eat_floor_item(int item_link);
 
-bool eat_from_floor(void);
+int eat_from_floor();
 
 void eat_from_inventory(int which_inventory_slot);
 
