@@ -3,7 +3,7 @@
  *  Summary: deal with reading and writing of highscore file
  *  Written by: Gordon Lipford
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T12:44:24.789033Z $
  *
  *  Change History (most recent first):
  *
@@ -216,16 +216,9 @@ static void hiscores_print_entry(const scorefile_entry &se,
     std::string entry;
     // format the entry
     if (format == SCORE_TERSE)
-    {
         entry = hiscores_format_single( se );
-        // truncate if we want short format
-        if (entry.length() > 80)
-            entry = entry.substr(0, 80);
-    }
     else
-    {
         entry = hiscores_format_single_long( se, (format == SCORE_VERBOSE) );
-    }
 
     entry += EOL;
     pf("%s", entry.c_str());
@@ -921,8 +914,7 @@ void scorefile_entry::init()
     for (int i = 0; i < NUM_RUNE_TYPES; i++)
         rune_array[i] = 0;
 
-    const bool calc_item_values =
-        (death_type == KILLED_BY_LEAVING || death_type == KILLED_BY_WINNING);
+    const bool calc_item_values = death_type == KILLED_BY_WINNING;
     
     // Calculate value of pack and runes when character leaves dungeon
     for (int d = 0; d < ENDOFPACK; d++)
