@@ -3,7 +3,7 @@
  *  Summary:    Simple reading of init file.
  *  Written by: David Loewenstern
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-09-24T20:12:15.974817Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T12:50:51.678403Z $
  *
  *  Change History (most recent first):
  *
@@ -38,7 +38,8 @@ std::string read_init_file(bool runscript = false);
 
 void read_startup_prefs();
 void read_options(FILE *f, bool runscript = false);
-void read_options(const std::string &s, bool runscript = false);
+void read_options(const std::string &s, bool runscript = false,
+                  bool clear_aliases = false);
 
 void parse_option_line(const std::string &line, bool runscript = false);
 
