@@ -15,12 +15,42 @@
 
 #include "externs.h"
 
+enum potion_type
+{
+    POT_HEALING,                       //    0
+    POT_HEAL_WOUNDS,
+    POT_SPEED,
+    POT_MIGHT,
+    POT_GAIN_STRENGTH,
+    POT_GAIN_DEXTERITY,                //    5
+    POT_GAIN_INTELLIGENCE,
+    POT_LEVITATION,
+    POT_POISON,
+    POT_SLOWING,
+    POT_PARALYSIS,                     //   10
+    POT_CONFUSION,
+    POT_INVISIBILITY,
+    POT_PORRIDGE,
+    POT_DEGENERATION,
+    POT_DECAY,                         //   15
+    POT_WATER,
+    POT_EXPERIENCE,
+    POT_MAGIC,
+    POT_RESTORE_ABILITIES,
+    POT_STRONG_POISON,                 //   20
+    POT_BERSERK_RAGE,
+    POT_CURE_MUTATION,
+    POT_MUTATION,
+    POT_BLOOD,
+    POT_RESISTANCE,
+    NUM_POTIONS
+};
 
 /* ***********************************************************************
  * called from: ability - beam - decks - item_use - misc - religion -
  *              spell - spells - spells1
  * *********************************************************************** */
-bool potion_effect(char pot_eff, int pow);
+bool potion_effect(potion_type pot_eff, int pow);
 
 
 /* ***********************************************************************
@@ -39,6 +69,6 @@ void unwear_armour(char unw);
 /* ***********************************************************************
  * called from: decks - it_use3 - item_use - items - spells3 - transfor
  * *********************************************************************** */
-void unwield_item(char unw, bool showMsgs = true);
+void unwield_item(bool showMsgs = true);
 
 #endif
