@@ -3,7 +3,7 @@
  *  Summary:    Functions for using some of the wackier inventory items.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:10:01.706311Z $
  *
  *  Change History (most recent first):
  *
@@ -248,9 +248,9 @@ void special_wielded()
     return;
 }                               // end special_wielded()
 
-static void reaching_weapon_attack(void)
+static bool reaching_weapon_attack(void)
 {
-    struct dist beam;
+    dist beam;
     int x_distance, y_distance;
     int x_middle, y_middle;
     int skill;
@@ -259,68 +259,74 @@ static void reaching_weapon_attack(void)
 
     direction(beam, DIR_TARGET, TARG_ENEMY);
     if (!beam.isValid)
-        return;
+        return false;
 
     if (beam.isMe)
     {
         canned_msg(MSG_UNTHINKING_ACT);
-        return;
+        return false;
     }
 
     x_distance = abs(beam.tx - you.x_pos);
     y_distance = abs(beam.ty - you.y_pos);
 
     if (x_distance > 2 || y_distance > 2)
+    {
         mpr("Your weapon cannot reach that far!");
+        return false;
+    }
     else if (mgrd[beam.tx][beam.ty] == NON_MONSTER)
     {
+        // Must return true, otherwise you get a free discovery
+        // of invisible monsters. Maybe we shouldn't do practice
+        // here to prevent scumming...but that would just encourage
+        // finding popcorn monsters.
         mpr("You attack empty space.");
+        return true;
     }
-    else
+
+    /* BCR - Added a check for monsters in the way.  Only checks cardinal
+     *       directions.  Knight moves are ignored.  Assume the weapon
+     *       slips between the squares.
+     */
+
+    // if we're attacking more than a space away
+    if ((x_distance > 1) || (y_distance > 1))
     {
-        /* BCR - Added a check for monsters in the way.  Only checks cardinal
-         *       directions.  Knight moves are ignored.  Assume the weapon
-         *       slips between the squares.
-         */
+        x_middle = MAX(beam.tx, you.x_pos) - (x_distance / 2);
+        y_middle = MAX(beam.ty, you.y_pos) - (y_distance / 2);
 
-        // if we're attacking more than a space away
-        if ((x_distance > 1) || (y_distance > 1))
+        // if either the x or the y is the same, we should check for
+        // a monster:
+        if (((beam.tx == you.x_pos) || (beam.ty == you.y_pos))
+                && (mgrd[x_middle][y_middle] != NON_MONSTER))
         {
-            x_middle = MAX(beam.tx, you.x_pos) - (x_distance / 2);
-            y_middle = MAX(beam.ty, you.y_pos) - (y_distance / 2);
+            skill = weapon_skill( you.inv[you.equip[EQ_WEAPON]].base_type,
+                                  you.inv[you.equip[EQ_WEAPON]].sub_type );
 
-            // if either the x or the y is the same, we should check for
-            // a monster:
-            if (((beam.tx == you.x_pos) || (beam.ty == you.y_pos))
-                    && (mgrd[x_middle][y_middle] != NON_MONSTER))
-            {
-                skill = weapon_skill( you.inv[you.equip[EQ_WEAPON]].base_type,
-                                      you.inv[you.equip[EQ_WEAPON]].sub_type );
-
-                if ((5 + (3 * skill)) > random2(100))
-                {
-                    mpr("You reach to attack!");
-                    you_attack(mgrd[beam.tx][beam.ty], false);
-                }
-                else
-                {
-                    mpr("You could not reach far enough!");
-                    you_attack(mgrd[x_middle][y_middle], false);
-                }
-            }
-            else
+            if ((5 + (3 * skill)) > random2(40))
             {
                 mpr("You reach to attack!");
                 you_attack(mgrd[beam.tx][beam.ty], false);
             }
+            else
+            {
+                mpr("You could not reach far enough!");
+                return true;
+            }
         }
         else
         {
+            mpr("You reach to attack!");
             you_attack(mgrd[beam.tx][beam.ty], false);
         }
     }
+    else
+    {
+        you_attack(mgrd[beam.tx][beam.ty], false);
+    }
 
-    return;
+    return true;
 }                               // end reaching_weapon_attack()
 
 // returns true if item successfully evoked.
@@ -358,15 +364,15 @@ bool evoke_wielded( void )
     switch (wpn.base_type)
     {
     case OBJ_WEAPONS:
-        if (get_weapon_brand( wpn ) == SPWPN_REACHING
-            && enough_mp(1, false))
+        if (get_weapon_brand(wpn) == SPWPN_REACHING)
         {
-            // needed a cost to prevent evocation training abuse -- bwr
-            dec_mp(1);
-            make_hungry( 50, false );  
-            reaching_weapon_attack();
-            pract = (one_chance_in(5) ? 1 : 0);
-            did_work = true;
+            if (reaching_weapon_attack())
+            {
+                pract = 0;
+                did_work = true;
+            }
+            else
+                return false;
         }
         else if (is_fixed_artefact( wpn ))
         {
