@@ -3,7 +3,7 @@
  *  Summary:    Misc functions.
  *  Written by: Brent Ross
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-06T05:56:50.150565Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -24,6 +24,8 @@
 
 #include "externs.h"
 
+#include "decks.h"
+#include "food.h"
 #include "items.h"
 #include "itemprop.h"
 #include "macro.h"
@@ -90,30 +92,30 @@ static armour_def Armour_prop[NUM_ARMOURS] =
         true,  EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
     { ARM_STEAM_DRAGON_ARMOUR,  "steam dragon armour",    4,  0,  120, 
         true,  EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_MOTTLED_DRAGON_HIDE,  "mottled dragon hide",    2, -1,  150, 
+    { ARM_MOTTLED_DRAGON_HIDE,  "mottled dragon hide",    3, -1,  150, 
         true,  EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
     { ARM_MOTTLED_DRAGON_ARMOUR,"mottled dragon armour",  5, -1,  150, 
         true,  EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
 
     { ARM_SWAMP_DRAGON_HIDE,    "swamp dragon hide",      3, -2,  200, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_SWAMP_DRAGON_ARMOUR,  "swamp dragon armour",    5, -2,  200, 
+    { ARM_SWAMP_DRAGON_ARMOUR,  "swamp dragon armour",    7, -2,  200, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
     { ARM_DRAGON_HIDE,          "dragon hide",            3, -3,  350, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_DRAGON_ARMOUR,        "dragon armour",          6, -3,  350, 
+    { ARM_DRAGON_ARMOUR,        "dragon armour",          8, -3,  350, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_ICE_DRAGON_HIDE,      "ice dragon hide",        3, -3,  350, 
+    { ARM_ICE_DRAGON_HIDE,      "ice dragon hide",        4, -3,  350, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_ICE_DRAGON_ARMOUR,    "ice dragon armour",      6, -3,  350, 
+    { ARM_ICE_DRAGON_ARMOUR,    "ice dragon armour",      9, -3,  350, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_STORM_DRAGON_HIDE,    "storm dragon hide",      4, -5,  600, 
+    { ARM_STORM_DRAGON_HIDE,    "storm dragon hide",      4, -4,  600, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_STORM_DRAGON_ARMOUR,  "storm dragon armour",    8, -5,  600, 
+    { ARM_STORM_DRAGON_ARMOUR,  "storm dragon armour",    10, -5,  600, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_GOLD_DRAGON_HIDE,     "gold dragon hide",       5, -8, 1100, 
+    { ARM_GOLD_DRAGON_HIDE,     "gold dragon hide",       5, -5, 1100, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
-    { ARM_GOLD_DRAGON_ARMOUR,   "gold dragon armour",   10, -8, 1100, 
+    { ARM_GOLD_DRAGON_ARMOUR,   "gold dragon armour",   13, -9, 1100, 
         false, EQ_BODY_ARMOUR, SIZE_LITTLE, SIZE_GIANT },
 
     { ARM_CLOAK,                "cloak",                  1,  0,   40, 
@@ -343,12 +345,15 @@ struct missile_def
 static int Missile_index[NUM_MISSILES];
 static missile_def Missile_prop[NUM_MISSILES] = 
 {
-    { MI_NEEDLE,        "needle",       0,    1, false },
-    { MI_STONE,         "stone",        4,    2, true  },
-    { MI_DART,          "dart",         5,    3, true  },
-    { MI_ARROW,         "arrow",        6,    5, false },
-    { MI_BOLT,          "bolt",         8,    5, false },
-    { MI_LARGE_ROCK,    "large rock",  20, 1000, true  },
+    { MI_NEEDLE,        "needle",        0,    1, false },
+    { MI_STONE,         "stone",         4,    2, true  },
+    { MI_DART,          "dart",          5,    3, true  },
+    { MI_ARROW,         "arrow",         7,    5, false },
+    { MI_BOLT,          "bolt",          9,    5, false },
+    { MI_LARGE_ROCK,    "large rock",   20,  600, true  },
+    { MI_SLING_BULLET,  "sling bullet",  6,    4, false },
+    { MI_JAVELIN,       "javelin",      10,   80, true  },
+    { MI_THROWING_NET,  "throwing net",  0,   30, true  }, 
 };
 
 struct food_def 
@@ -398,7 +403,7 @@ static food_def Food_prop[NUM_FOODS] =
 
 // Must call this functions early on so that the above tables can
 // be accessed correctly.
-void init_properties(void)
+void init_properties()
 {
     int i;
 
@@ -448,6 +453,28 @@ void do_uncurse_item( item_def &item )
     item.flags &= (~ISFLAG_CURSED);
 }
 
+//
+// Is item stationary (cannot be picked up?)
+//
+void set_item_stationary( item_def &item )
+{
+    if (item.base_type == OBJ_MISSILES && item.sub_type == MI_THROWING_NET)
+        item.plus2 = 1;
+}
+
+void remove_item_stationary( item_def &item )
+{
+    if (item.base_type == OBJ_MISSILES && item.sub_type == MI_THROWING_NET)
+        item.plus2 = 0;
+}
+
+bool item_is_stationary( const item_def &item )
+{
+    return (item.base_type == OBJ_MISSILES
+            && item.sub_type == MI_THROWING_NET
+            && item.plus2);
+}
+
 //
 // Item identification status:
 //
@@ -456,19 +483,38 @@ bool item_ident( const item_def &item, unsigned long flags )
     return ((item.flags & flags) == flags);
 }
 
+// The Orb of Zot and unique runes are considered critical.
+bool item_is_critical(const item_def &item)
+{
+    if (!is_valid_item(item))
+        return (false);
+    
+    if (item.base_type == OBJ_ORBS)
+        return (true);
+
+    return (item.base_type == OBJ_MISCELLANY
+            && item.sub_type == MISC_RUNE_OF_ZOT
+            && (item.plus != RUNE_DEMONIC
+                && item.plus != RUNE_ABYSSAL));
+}
+
 void set_ident_flags( item_def &item, unsigned long flags )
 {
-    bool known_before = fully_identified(item);
-    item.flags |= flags;
-    if ( !known_before && fully_identified(item) ) {
-        /* make a note of it */
-        if ( is_interesting_item(item) ) {
-            char buf[ITEMNAME_SIZE];
-            char buf2[ITEMNAME_SIZE];
-            item_name( item, DESC_NOCAP_A, buf );
-            strcpy(buf2, origin_desc(item).c_str());
-            take_note(Note(NOTE_ID_ITEM, 0, 0, buf, buf2));
-        }
+    if ((item.flags & flags) != flags)
+    {
+        item.flags |= flags;
+        request_autoinscribe();
+    }
+
+    if (notes_are_active() && !(item.flags & ISFLAG_NOTED_ID) &&
+        fully_identified(item) && is_interesting_item(item))
+    {
+        // make a note of it
+        take_note(Note(NOTE_ID_ITEM, 0, 0, item.name(DESC_NOCAP_A).c_str(),
+                       origin_desc(item).c_str()));
+        // sometimes (e.g. shops) you can ID an item before you get it;
+        // don't note twice in those cases
+        item.flags |= (ISFLAG_NOTED_ID | ISFLAG_NOTED_GET);
     }
 }
 
@@ -477,6 +523,63 @@ void unset_ident_flags( item_def &item, unsigned long flags )
     item.flags &= (~flags);
 }
 
+// Returns the mask of interesting identify bits for this item
+// (e.g., scrolls don't have know-cursedness.)
+unsigned long full_ident_mask( const item_def& item )
+{
+    unsigned long flagset = ISFLAG_IDENT_MASK;
+    switch ( item.base_type )
+    {
+    case OBJ_FOOD:
+        flagset = 0;
+        break;
+    case OBJ_MISCELLANY:
+        if ( item.sub_type == MISC_RUNE_OF_ZOT )
+            flagset = 0;
+        else
+            flagset = ISFLAG_KNOW_TYPE;
+        break;
+    case OBJ_BOOKS:
+    case OBJ_ORBS:
+    case OBJ_SCROLLS:
+    case OBJ_POTIONS:
+    case OBJ_STAVES:
+        flagset = ISFLAG_KNOW_TYPE;
+        break;
+    case OBJ_WANDS:
+        flagset = (ISFLAG_KNOW_TYPE | ISFLAG_KNOW_PLUSES);
+        break;
+    case OBJ_JEWELLERY:
+        flagset = (ISFLAG_KNOW_CURSE | ISFLAG_KNOW_TYPE);
+        if ( ring_has_pluses(item) )
+            flagset |= ISFLAG_KNOW_PLUSES;
+        break;
+    case OBJ_MISSILES:
+        flagset = ISFLAG_KNOW_PLUSES | ISFLAG_KNOW_TYPE;
+        if (get_ammo_brand(item) == SPMSL_NORMAL)
+            flagset &= ~ISFLAG_KNOW_TYPE;
+        break;
+    case OBJ_WEAPONS:
+    case OBJ_ARMOUR:
+    default:
+        break;
+    }
+
+    if (item_type_known(item))
+        flagset &= (~ISFLAG_KNOW_TYPE);
+
+    if ( is_artefact(item) )
+    {
+        flagset |= ISFLAG_KNOW_PROPERTIES;
+    }
+    return flagset;
+}
+
+bool fully_identified( const item_def& item )
+{
+    return item_ident(item, full_ident_mask(item));
+}
+
 //
 // Equipment race and description:
 //
@@ -525,8 +628,11 @@ void set_equip_race( item_def &item, unsigned long flags )
         break;
 
     case OBJ_MISSILES:
-        if (item.sub_type == MI_STONE || item.sub_type == MI_LARGE_ROCK)
+        if (item.sub_type == MI_STONE || item.sub_type == MI_LARGE_ROCK
+            || item.sub_type == MI_THROWING_NET)
+        {
             return;
+        }
         break;
 
     default:
@@ -713,7 +819,7 @@ int get_ammo_brand( const item_def &item )
     return (item.special);
 }
 
-int get_armour_ego_type( const item_def &item )
+special_armour_type get_armour_ego_type( const item_def &item )
 {
     // artefact armours have no ego type, must look up powers separately
     if (item.base_type != OBJ_ARMOUR 
@@ -722,7 +828,7 @@ int get_armour_ego_type( const item_def &item )
         return (SPARM_NORMAL);
     }
 
-    return (item.special);
+    return (static_cast<special_armour_type>(item.special));
 }
 
 // 
@@ -888,6 +994,11 @@ equipment_type get_armour_slot( const item_def &item )
     return (Armour_prop[ Armour_index[item.sub_type] ].slot);
 }
 
+equipment_type get_armour_slot( armour_type arm )
+{
+    return (Armour_prop[ Armour_index[arm] ].slot);
+}
+
 bool jewellery_is_amulet( const item_def &item )
 {
     ASSERT( item.base_type == OBJ_JEWELLERY );
@@ -953,6 +1064,15 @@ bool check_armour_shape( const item_def &item, bool quiet )
         switch (slot)
         {
         case EQ_BOOTS:
+
+            if (you.mutation[MUT_HOOVES])
+            {
+                if (!quiet)
+                    mpr("You can't wear boots with hooves!");
+
+                return (false);
+            }
+
             switch (you.species)
             {
             case SP_NAGA:
@@ -1004,7 +1124,7 @@ bool check_armour_shape( const item_def &item, bool quiet )
                     return (false);
                 }
 
-                if (you.mutation[MUT_HOOVES] >= 2)
+                if (you.mutation[MUT_HOOVES])
                 {
                     if (!quiet)
                         mpr( "You can't wear boots with hooves!" );
@@ -1219,7 +1339,8 @@ bool does_damage_type( const item_def &item, int dam_type )
 }                               // end does_damage_type()
 
 
-hands_reqd_type hands_reqd(int base_type, int sub_type, size_type size)
+hands_reqd_type hands_reqd(object_class_type base_type, int sub_type,
+                           size_type size)
 {
     item_def item;
     item.base_type = base_type;
@@ -1259,7 +1380,7 @@ hands_reqd_type hands_reqd( const item_def &item, size_type size )
         if (ret == HANDS_DOUBLE)
         {
             doub = true;
-            ret = HANDS_HALF;
+            ret = HANDS_TWO; // HANDS_HALF once double-ended is implemented
         }
 
         // adjust handedness for size only for non-whip melee weapons
@@ -1397,7 +1518,7 @@ skill_type weapon_skill( const item_def &item )
 }
 
 // front function for the above when we don't have a physical item to check
-skill_type weapon_skill( int wclass, int wtype )
+skill_type weapon_skill( object_class_type wclass, int wtype )
 {
     item_def    wpn;
 
@@ -1412,14 +1533,21 @@ skill_type range_skill( const item_def &item )
 {
     if (item.base_type == OBJ_WEAPONS && is_range_weapon( item ))
         return (Weapon_prop[ Weapon_index[item.sub_type] ].skill);
-    else if (item.base_type == OBJ_MISSILES && item.sub_type == MI_DART)
-        return (SK_DARTS);
+    else if (item.base_type == OBJ_MISSILES)
+    {
+        switch (item.sub_type)
+        {
+        case MI_DART:    return (SK_DARTS);
+        case MI_JAVELIN: return (SK_POLEARMS);
+        default:         break;
+        }
+    }
 
-    return (SK_RANGED_COMBAT);
+    return (SK_THROWING);
 }
 
 // front function for the above when we don't have a physical item to check
-skill_type range_skill( int wclass, int wtype )
+skill_type range_skill( object_class_type wclass, int wtype )
 {
     item_def    wpn;
 
@@ -1638,6 +1766,21 @@ bool is_range_weapon( const item_def &item )
     return (fires_ammo_type( item ) != MI_NONE);
 }
 
+// decide if something is launched or thrown
+launch_retval is_launched(actor *actor, const item_def *launcher,
+                         const item_def &missile)
+{
+    if (missile.base_type == OBJ_MISSILES
+        && launcher
+        && missile.launched_by(*launcher))
+    {
+        return (LRET_LAUNCHED);
+    }
+
+    return (is_throwable(missile, actor->body_size())?
+            LRET_THROWN : LRET_FUMBLED);
+}
+
 bool is_range_weapon_type( weapon_type wtype )
 {
     item_def wpn;
@@ -1648,93 +1791,46 @@ bool is_range_weapon_type( weapon_type wtype )
     return (is_range_weapon( wpn ));
 }
 
+const char *ammo_name(missile_type ammo)
+{
+    return (ammo < 0 || ammo >= NUM_MISSILES? "eggplant"
+            : Missile_prop[ Missile_index[ammo] ].name);
+}
+
 const char * ammo_name( const item_def &bow )
 {
     ASSERT( is_range_weapon( bow ) );
-
-    const int ammo = fires_ammo_type( bow );
-
-    return (Missile_prop[ Missile_index[ammo] ].name);
+    return ammo_name(fires_ammo_type( bow ));
 } 
 
 // returns true if item can be reasonably thrown without a launcher
-bool is_throwable( const item_def &wpn )
+bool is_throwable( const item_def &wpn, size_type bodysize  )
 {
     if (wpn.base_type == OBJ_WEAPONS)
         return (Weapon_prop[ Weapon_index[wpn.sub_type] ].throwable);
     else if (wpn.base_type == OBJ_MISSILES)
+    {
+        if (bodysize < SIZE_MEDIUM && wpn.sub_type == MI_JAVELIN)
+            return (false);
+        if (bodysize < SIZE_MEDIUM && wpn.sub_type == MI_THROWING_NET)
+            return (false);
         return (Missile_prop[ Missile_index[wpn.sub_type] ].throwable);
+    }
 
     return (false);
 }
 
-// FIXME
-#if 0
-// decide if "being" is launching or throwing "ammo"
-launch_retval is_launched( int being_id, const item_def &ammo, bool msg )
-{
-    ASSERT( being_id != MHITNOT );
-
-    launch_retval  ret = LRET_FUMBLED;
-
-    const item_def * lnch = 0;
-    int              fit = 0;
-
-    if (being_id == MHITYOU)
-    {
-        const int wpn = get_inv_wielded();
-        lnch = (wpn == -1) ? 0 : &you.inv[wpn];
-        fit = fit_item_throwable_size( ammo, player_size() );
-    }
-    else // monster case
-    {
-        const int wpn = menv[being_id].inv[MSLOT_WEAPON]; 
-        lnch = (wpn == NON_ITEM) ? 0 : &mitm[wpn];
-        fit = fit_item_throwable_size( ammo, mons_size( menv[being_id].type ) );
-    }
-
-    if (lnch 
-        && lnch->base_type == OBJ_WEAPONS
-        && is_range_weapon( *lnch )
-        && ammo.base_type == OBJ_MISSILES
-        && ammo.sub_type == fires_ammo_type( *lnch ))
-    {
-        ret = LRET_LAUNCHED;
-    }
-    else if (is_throwable( ammo ))
-    {
-        if (fit == 0)
-            ret = LRET_THROWN;
-        else 
-        {
-            ret = LRET_FUMBLED;
-
-            if (being_id == MHITYOU && msg)
-            {
-                mpr( MSGCH_WARN, "It's difficult to throw such a%s object.",
-                     (fit > 0) ? " large" : (fit < 0) ? " small" : "n awkward" );
-            }
-        }
-    }
-
-    return (ret);
-}
-#endif
-
 //
 // Staff/rod functions:
 //
 bool item_is_rod( const item_def &item )
 {
-    return (item.base_type == OBJ_STAVES 
-            && item.sub_type >= STAFF_SMITING && item.sub_type < STAFF_AIR);
+    return (item.base_type == OBJ_STAVES && item.sub_type >= STAFF_SMITING);
 }
 
 bool item_is_staff( const item_def &item )
 {
-    // Isn't De Morgan's law wonderful. -- bwr
-    return (item.base_type == OBJ_STAVES 
-            && (item.sub_type < STAFF_SMITING || item.sub_type >= STAFF_AIR));
+    return (item.base_type == OBJ_STAVES && !item_is_rod(item));
 }
 
 //
@@ -1871,6 +1967,186 @@ int property( const item_def &item, int prop_type )
     return (0);
 }
 
+bool gives_ability( const item_def &item )
+{
+    if (!item_type_known(item))
+        return false;
+
+    switch (item.base_type)
+    {
+       case OBJ_WEAPONS:
+       {
+          // unwielded weapon
+          item_def *weap = you.slot_item(EQ_WEAPON);
+          if (!weap || (*weap).slot != item.slot)
+              return false;
+          break;
+       }
+       case OBJ_JEWELLERY:
+       {
+          if (item.sub_type < NUM_RINGS)
+          {
+              // unworn ring
+              item_def *lring = you.slot_item(EQ_LEFT_RING);
+              item_def *rring = you.slot_item(EQ_RIGHT_RING);
+              if ((!lring || (*lring).slot != item.slot)
+                  && (!rring || (*rring).slot != item.slot))
+              {
+                  return false;
+              }
+              
+              if (item.sub_type == RING_TELEPORTATION
+                  || item.sub_type == RING_LEVITATION
+                  || item.sub_type == RING_INVISIBILITY)
+              {
+                  return true;
+              }
+          }
+          else
+          {
+              // unworn amulet
+              item_def *amul = you.slot_item(EQ_AMULET);
+              if (!amul || (*amul).slot != item.slot)
+                  return false;
+                  
+              if (item.sub_type == AMU_RAGE)
+                  return true;
+          }
+          break;
+       }
+       case OBJ_ARMOUR:
+       {
+          const equipment_type eq = get_armour_slot(item);
+          if (eq == EQ_NONE)
+              return false;
+              
+          // unworn armour
+          item_def *arm = you.slot_item(eq);
+          if (!arm || (*arm).slot != item.slot)
+              return false;
+          break;
+       }
+       default:
+          return false;
+    }
+    
+    if (!is_random_artefact(item))
+        return false;
+
+    // check for evokable randart properties
+    for (int rap = RAP_INVISIBLE; rap <= RAP_MAPPING; rap++)
+    {
+       if (randart_wpn_property( item, rap ))
+           return true;
+    }
+       
+    return false;
+}
+
+bool gives_resistance( const item_def &item )
+{
+    if (!item_type_known(item))
+        return false;
+
+    switch (item.base_type)
+    {
+       case OBJ_WEAPONS:
+       {
+          // unwielded weapon
+          item_def *weap = you.slot_item(EQ_WEAPON);
+          if (!weap || (*weap).slot != item.slot)
+              return false;
+          break;
+       }
+       case OBJ_JEWELLERY:
+       {
+          if (item.sub_type < NUM_RINGS)
+          {
+              // unworn ring
+              item_def *lring = you.slot_item(EQ_LEFT_RING);
+              item_def *rring = you.slot_item(EQ_RIGHT_RING);
+              if ((!lring || (*lring).slot != item.slot)
+                  && (!rring || (*rring).slot != item.slot))
+              {
+                  return false;
+              }
+
+              if (item.sub_type >= RING_PROTECTION_FROM_FIRE
+                  && item.sub_type <= RING_PROTECTION_FROM_COLD
+                  || item.sub_type == RING_SEE_INVISIBLE
+                  || item.sub_type >= RING_LIFE_PROTECTION
+                  && item.sub_type <= RING_TELEPORT_CONTROL
+                  || item.sub_type == RING_TELEPORTATION
+                  || item.sub_type == RING_SUSTAIN_ABILITIES)
+              {
+                  return true;
+              }
+          }
+          else
+          {
+              // unworn amulet
+              item_def *amul = you.slot_item(EQ_AMULET);
+              if (!amul || (*amul).slot != item.slot)
+                  return false;
+
+              if (item.sub_type != AMU_RAGE && item.sub_type != AMU_INACCURACY)
+                  return true;
+          }
+          break;
+       }
+       case OBJ_ARMOUR:
+       {
+          const equipment_type eq = get_armour_slot(item);
+          if (eq == EQ_NONE)
+              return false;
+
+          // unworn armour
+          item_def *arm = you.slot_item(eq);
+          if (!arm || (*arm).slot != item.slot)
+              return false;
+          break;
+          
+          const int ego = get_armour_ego_type( item );
+          if (ego >= SPARM_FIRE_RESISTANCE && ego <= SPARM_SEE_INVISIBLE
+              || ego == SPARM_RESISTANCE || ego == SPARM_POSITIVE_ENERGY)
+          {
+              return true;
+          }
+       }
+       case OBJ_STAVES:
+       {
+          // unwielded weapon
+          item_def *weap = you.slot_item(EQ_WEAPON);
+          if (!weap || (*weap).slot != item.slot)
+              return false;
+          
+          if (item.sub_type >= STAFF_FIRE && item.sub_type <= STAFF_POISON
+              || item.sub_type == STAFF_AIR)
+          {
+              return true;
+          }
+          return false;
+       }
+       default:
+          return false;
+    }
+
+    if (!is_random_artefact(item))
+        return false;
+
+    // check for randart resistances
+    for (int rap = RAP_FIRE; rap <= RAP_CAN_TELEPORT; rap++)
+    {
+       if (rap == RAP_MAGIC || rap >= RAP_INVISIBLE && rap != RAP_CAN_TELEPORT)
+           continue;
+           
+       if (randart_wpn_property( item, rap ))
+           return true;
+    }
+
+    return false;
+}
+
 int item_mass( const item_def &item )
 {
     int unit_mass = 0;
@@ -1939,6 +2215,11 @@ int item_mass( const item_def &item )
         break;
 
     case OBJ_MISCELLANY:
+        if ( is_deck(item) )
+        {
+            unit_mass = 50;
+            break;
+        }
         switch (item.sub_type)
         {
         case MISC_BOTTLED_EFREET:           
@@ -2015,10 +2296,6 @@ size_type item_size( const item_def &item )
         break;
 
     case OBJ_MISCELLANY:
-        if (item.sub_type == MISC_PORTABLE_ALTAR_OF_NEMELEX)
-        {
-            size = SIZE_SMALL;
-        }
         break;
 
     case OBJ_CORPSES:
@@ -2085,3 +2362,58 @@ bool is_shield_incompatible(const item_def &weapon, const item_def *shield)
             && !item_is_rod(weapon) 
             && !is_range_weapon(weapon);
 }
+
+std::string item_base_name(const item_def &item)
+{
+    switch ( item.base_type )
+    {
+    case OBJ_WEAPONS:
+        return Weapon_prop[Weapon_index[item.sub_type]].name;
+    case OBJ_MISSILES:
+        return Missile_prop[Missile_index[item.sub_type]].name;
+    case OBJ_ARMOUR:
+        if ( item.sub_type != ARM_HELMET )
+        {
+            return Armour_prop[Armour_index[item.sub_type]].name;
+        }
+        else
+        {
+            std::string result;
+            if (get_helmet_type(item) == THELM_HELM ||
+                get_helmet_type(item) == THELM_HELMET) 
+            {
+                const short dhelm = get_helmet_desc( item );
+                
+                result +=
+                    (dhelm == THELM_DESC_PLAIN)    ? "" :
+                    (dhelm == THELM_DESC_WINGED)   ? "winged " :
+                    (dhelm == THELM_DESC_HORNED)   ? "horned " :
+                    (dhelm == THELM_DESC_CRESTED)  ? "crested " :
+                    (dhelm == THELM_DESC_PLUMED)   ? "plumed " :
+                    (dhelm == THELM_DESC_SPIKED)   ? "spiked " :
+                    (dhelm == THELM_DESC_VISORED)  ? "visored " :
+                    (dhelm == THELM_DESC_JEWELLED) ? "jewelled "
+                                                   : "buggy ";
+            }
+            
+            const short helm_type = get_helmet_type( item );
+            if (helm_type == THELM_HELM)
+                result += "helm";
+            else if (helm_type == THELM_CAP)
+                result += "cap";
+            else if (helm_type == THELM_WIZARD_HAT)
+                result += "wizard's hat";
+            else 
+                result += "helmet";
+            
+            return result;
+        }
+    default:
+        return "";
+    }
+}
+
+const char* weapon_base_name(unsigned char subtype)
+{
+    return Weapon_prop[Weapon_index[subtype]].name;
+}
