@@ -3,7 +3,7 @@
  *  Summary:    Misc (mostly) inventory related functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T12:54:25.060870Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:12:32.043229Z $
  *
  *  Change History (most recent first):
  *
@@ -2824,6 +2824,7 @@ static void do_autopickup()
     
     o = igrd[you.x_pos][you.y_pos];
 
+    std::string pickup_warning;
     while (o != NON_ITEM)
     {
         next = mitm[o].link;
@@ -2862,19 +2863,21 @@ static void do_autopickup()
             {
                 n_tried_pickup++;
                 if (result == 0)
-                    mpr("You can't carry any more.");
+                    pickup_warning = "You can't carry any more.";
                 else
-                    mpr("Your pack is full.");
+                    pickup_warning = "Your pack is full.";
                 mitm[o].flags = iflags;
-                break;
             }
-
-            n_did_pickup++;
+            else
+                n_did_pickup++;
         }
 
         o = next;
     }
 
+    if (!pickup_warning.empty())
+        mpr(pickup_warning.c_str());
+
     if (n_did_pickup)
         you.turn_is_over = true;
 
