@@ -3,7 +3,7 @@
  *  Summary:    Functions for unix and curses support
  *  Written by: ?
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-11-24T17:24:21.713648Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-04T13:13:40.045599Z $
  *
  *  Change History (most recent first):
  *
@@ -600,7 +600,7 @@ int window(int x1, int y1, int x2, int y2)
 // textcolour and drawing a character and call set_altcharset(false)
 // after you're done drawing.
 //
-int cset_adjust(int raw)
+unsigned cset_adjust(unsigned raw)
 {
     if (Options.char_set != CSET_ASCII && Options.char_set != CSET_UNICODE)
     {
@@ -617,6 +617,13 @@ int cset_adjust(int raw)
     return (raw);
 }
 
+void put_colour_ch(int colour, unsigned ch)
+{
+    ch = cset_adjust(ch);
+    textattr(colour);
+    putwch(ch);
+}
+
 void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
 {
     for (int y = y1; y <= y2; ++y)
@@ -624,13 +631,12 @@ void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
         gotoxy(x1, y);
         for (int x = x1; x <= x2; ++x)
         {
-            const screen_buffer_t ch = cset_adjust( *buf );
+            const screen_buffer_t ch = cset_adjust(*buf);
             textattr( buf[1] );
             putwch( ch );
             buf += 2;
         }
     }
-    set_altcharset(false);
     update_screen();
 }
 
@@ -641,6 +647,7 @@ void puttext(int x1, int y1, int x2, int y2, const screen_buffer_t *buf)
 // C++ string class.  -- bwr
 void update_screen(void)
 {
+    set_altcharset(false);
     refresh();
 }
 
