@@ -31,6 +31,7 @@ int itoa(int value, char *strptr, int radix);
 int kbhit(void);
 int putch(unsigned char chr);
 int putwch(unsigned chr);
+void put_colour_ch(int colour, unsigned ch);
 int stricmp(const char *str1, const char *str2);
 int translate_keypad(int keyin);
 int wherex(void);
@@ -48,7 +49,6 @@ void unixcurses_startup(void);
 void textbackground(int bg);
 void textcolor(int col);
 void textattr(int col);
-int cset_adjust(int raw);
 
 void set_altcharset(bool alt_on);
 bool get_altcharset();
