@@ -1,5 +1,5 @@
 /*
-*  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-10-27T16:44:37.046050Z $
+*  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-03T21:06:37.928309Z $
 */
 
 #ifndef LIBW32C_H
@@ -54,4 +54,10 @@ void enable_smart_cursor(bool cursor);
 bool is_smart_cursor_enabled();
 void set_mouse_enabled(bool enabled);
 
+inline void put_colour_ch(int colour, unsigned ch)
+{
+    textattr(colour);
+    putwch(ch);
+}
+
 #endif
