@@ -2,7 +2,7 @@
  *  File:       luadgn.cc
  *  Summary:    Dungeon-builder Lua interface.
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-20T13:11:33.806266Z $
  */
 
 #include "AppHdr.h"
@@ -830,15 +830,14 @@ const char *dngn_feature_names[] =
     "unseen", "rock_wall", "stone_wall", "closed_door", "metal_wall",
     "secret_door", "green_crystal_wall", "orcish_idol", "wax_wall",
     "permarock_wall", "", "", "", "", "", "", "", "", "", "", "",
-    "silver_statue", "granite_statue", "orange_crystal_statue",
-    "statue_reserved_1", "statue_reserved_2", "", "", "", "", "", "",
+    "granite_statue", "statue_reserved_1", "statue_reserved_2", "",
     "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
-    "", "", "", "", "", "", "", "", "", "", "", "", "", "lava",
-    "deep_water", "", "", "shallow_water", "water_stuck", "floor",
-    "floor_special", "floor_reserved", "exit_hell", "enter_hell",
-    "open_door", "", "", "trap_mechanical", "trap_magical", "trap_iii",
-    "undiscovered_trap", "", "enter_shop", "enter_labyrinth",
-    "stone_stairs_down_i", "stone_stairs_down_ii",
+    "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
+    "", "", "", "", "lava", "deep_water", "", "", "shallow_water",
+    "water_stuck", "floor", "floor_special", "floor_reserved",
+    "exit_hell", "enter_hell", "open_door", "", "", "trap_mechanical",
+    "trap_magical", "trap_iii", "undiscovered_trap", "", "enter_shop",
+    "enter_labyrinth", "stone_stairs_down_i", "stone_stairs_down_ii",
     "stone_stairs_down_iii", "rock_stairs_down", "stone_stairs_up_i",
     "stone_stairs_up_ii", "stone_stairs_up_iii", "rock_stairs_up", "",
     "", "enter_dis", "enter_gehenna", "enter_cocytus",
@@ -859,17 +858,17 @@ const char *dngn_feature_names[] =
     "return_from_elven_halls", "return_from_tomb",
     "return_from_swamp", "return_from_shoals", "return_reserved_2",
     "return_reserved_3", "return_reserved_4", "", "", "", "", "", "",
-    "", "", "", "", "", "", "", "enter_portal_vault", "exit_portal_vault",
-    "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",
-    "", "", "altar_zin", "altar_shining_one", "altar_kikubaaqudgha",
-    "altar_yredelemnul", "altar_xom", "altar_vehumet",
-    "altar_okawaru", "altar_makhleb", "altar_sif_muna", "altar_trog",
-    "altar_nemelex_xobeh", "altar_elyvilon", "altar_lugonu",
-    "altar_beogh", "", "", "", "", "", "", "blue_fountain",
-    "dry_fountain_i", "sparkling_fountain", "dry_fountain_ii",
-    "dry_fountain_iii", "dry_fountain_iv", "dry_fountain_v",
-    "dry_fountain_vi", "dry_fountain_vii", "dry_fountain_viii",
-    "permadry_fountain"
+    "", "", "", "", "", "", "", "enter_portal_vault",
+    "exit_portal_vault", "", "", "", "", "", "", "", "", "", "", "",
+    "", "", "", "", "", "", "", "altar_zin", "altar_shining_one",
+    "altar_kikubaaqudgha", "altar_yredelemnul", "altar_xom",
+    "altar_vehumet", "altar_okawaru", "altar_makhleb",
+    "altar_sif_muna", "altar_trog", "altar_nemelex_xobeh",
+    "altar_elyvilon", "altar_lugonu", "altar_beogh", "", "", "", "",
+    "", "", "blue_fountain", "dry_fountain_i", "sparkling_fountain",
+    "dry_fountain_ii", "dry_fountain_iii", "dry_fountain_iv",
+    "dry_fountain_v", "dry_fountain_vi", "dry_fountain_vii",
+    "dry_fountain_viii", "permadry_fountain"
 };
 
 dungeon_feature_type dungeon_feature_by_name(const std::string &name)
