@@ -3,7 +3,7 @@
  *  Summary:    Functions used to print messages.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-06-05T16:54:50.325619Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -25,17 +25,21 @@
 
 #include "externs.h"
 
+#include "cio.h"
+#include "delay.h"
 #include "initfile.h"
 #include "libutil.h"
 #include "macro.h"
-#include "delay.h"
+#include "message.h"
+#include "monstuff.h"
+#include "notes.h"
+#include "religion.h"
+#include "stash.h"
+#include "state.h"
 #include "stuff.h"
 #include "travel.h"
-#include "view.h"
-#include "notes.h"
-#include "stash.h"
-#include "religion.h"
 #include "tutorial.h"
+#include "view.h"
 
 // circular buffer for keeping past messages
 message_item Store_Message[ NUM_STORED_MESSAGES ];    // buffer of old messages
@@ -45,7 +49,101 @@ int Message_Line = 0;                // line of next (previous?) message
 int New_Message_Count = 0;
 
 static bool suppress_messages = false;
-static void base_mpr(const char *inf, int channel, int param);
+static void base_mpr(const char *inf, msg_channel_type channel, int param);
+
+namespace msg
+{
+    std::ostream stream(new mpr_stream_buf(MSGCH_PLAIN));
+    std::vector<std::ostream*> stream_ptrs;
+    std::vector<mpr_stream_buf*> stream_buffers;
+
+    std::ostream& streams(msg_channel_type chan)
+    {
+        ASSERT(chan >= 0 &&
+               static_cast<unsigned int>(chan) < stream_ptrs.size());
+        return *stream_ptrs[chan];
+    }
+
+    void initialise_mpr_streams()
+    {
+        for (int i = 0; i < NUM_MESSAGE_CHANNELS; ++i)
+        {
+            mpr_stream_buf* pmsb =
+                new mpr_stream_buf(static_cast<msg_channel_type>(i));
+            std::ostream* pos = new std::ostream(pmsb);
+            (*pos) << std::nounitbuf;
+            stream_ptrs.push_back(pos);
+            stream_buffers.push_back(pmsb);
+        }
+        stream << std::nounitbuf;
+    }
+
+    void deinitalise_mpr_streams()
+    {
+        for (unsigned int i = 0; i < stream_ptrs.size(); ++i)
+            delete stream_ptrs[i];
+        stream_ptrs.clear();
+        for (unsigned int i = 0; i < stream_buffers.size(); ++i)
+            delete stream_buffers[i];
+        stream_buffers.clear();
+    }
+
+
+    setparam::setparam(int param)
+    {
+        m_param = param;
+    }
+
+    mpr_stream_buf::mpr_stream_buf(msg_channel_type chan) :
+        internal_count(0), param(0), muted(false), channel(chan)
+    {}
+
+    void mpr_stream_buf::set_param(int p)
+    {
+        this->param = p;
+    }
+
+    void mpr_stream_buf::set_muted(bool m)
+    {
+        this->muted = m;
+    }
+    
+    // again, can be improved
+    int mpr_stream_buf::overflow(int c)
+    {
+        if ( muted )
+            return 0;
+
+        if ( c == '\n' )
+        {
+            // null-terminate and print the string
+            internal_buf[internal_count] = 0;
+            mpr(internal_buf, channel, param);
+            
+            internal_count = 0;
+
+            // reset to defaults (param changing isn't sticky)
+            set_param(0);
+        }
+        else
+            internal_buf[internal_count++] = c;
+        
+        if ( internal_count + 3 > INTERNAL_LENGTH )
+        {
+            mpr("oops, hit overflow", MSGCH_DANGER);
+            internal_count = 0;
+            return std::streambuf::traits_type::eof();
+        }
+        return 0;
+    }
+}
+
+std::ostream& operator<<(std::ostream& os, const msg::setparam& sp)
+{
+    msg::mpr_stream_buf* ps = dynamic_cast<msg::mpr_stream_buf*>(os.rdbuf());
+    ps->set_param(sp.m_param);
+    return os;
+}
 
 no_messages::no_messages() : msuppressed(suppress_messages)
 {
@@ -57,7 +155,7 @@ no_messages::~no_messages()
     suppress_messages = msuppressed;
 }
 
-static char god_message_altar_colour( char god )
+static char god_message_altar_colour( god_type god )
 {
     int  rnd;
 
@@ -78,6 +176,9 @@ static char god_message_altar_colour( char god )
     case GOD_YREDELEMNUL:
         return (coinflip() ? DARKGREY : RED);
 
+    case GOD_BEOGH:
+        return (coinflip() ? BROWN : LIGHTRED);
+
     case GOD_KIKUBAAQUDGHA:
         return (DARKGREY);
 
@@ -105,18 +206,23 @@ static char god_message_altar_colour( char god )
     case GOD_SIF_MUNA:
         return (BLUE);
 
+    case GOD_LUGONU:
+        return (LIGHTRED);
+
     case GOD_NO_GOD:
-    default:
-        return(YELLOW);
+    case NUM_GODS:
+    case GOD_RANDOM:
+        return (YELLOW);
     }
+    return (YELLOW);            // for stupid compilers
 }
 
 #ifdef USE_COLOUR_MESSAGES
 
 // returns a colour or MSGCOL_MUTED
-int channel_to_colour( int channel, int param )
+int channel_to_colour( msg_channel_type channel, int param )
 {
-    char        ret;
+    char ret;
 
     switch (Options.channels[ channel ])
     {
@@ -138,8 +244,8 @@ int channel_to_colour( int channel, int param )
         case MSGCH_GOD:
         case MSGCH_PRAY:
             ret = (Options.channels[ channel ] == MSGCOL_DEFAULT)
-                                    ? god_colour( param )
-                                    : god_message_altar_colour( param );
+                ? god_colour( static_cast<god_type>(param) )
+                : god_message_altar_colour( static_cast<god_type>(param) );
             break;
 
         case MSGCH_DURATION:
@@ -167,6 +273,7 @@ int channel_to_colour( int channel, int param )
             break;
 
         case MSGCH_TALK:
+        case MSGCH_TALK_VISUAL:
             ret = WHITE;
             break;
 
@@ -202,6 +309,8 @@ int channel_to_colour( int channel, int param )
         case MSGCH_PLAIN:
         case MSGCH_ROTTEN_MEAT:
         case MSGCH_EQUIPMENT:
+        case MSGCH_EXAMINE:
+        case MSGCH_EXAMINE_FILTER:
         default:
             ret = param > 0? param : LIGHTGREY;
             break;
@@ -237,14 +346,14 @@ int channel_to_colour( int channel, int param )
 
 #else // don't use colour messages
 
-int channel_to_colour( int channel, int param )
+int channel_to_colour( msg_channel_type channel, int param )
 {
     return (LIGHTGREY);
 }
 
 #endif
 
-static void do_message_print( int channel, int param, 
+static void do_message_print( msg_channel_type channel, int param, 
                               const char *format, va_list argp )
 {
     char buff[200];
@@ -254,7 +363,15 @@ static void do_message_print( int channel, int param,
     mpr(buff, channel, param);
 }
 
-void mprf( int channel, const char *format, ... )
+void mprf( msg_channel_type channel, int param, const char *format, ... )
+{
+    va_list  argp;
+    va_start( argp, format );
+    do_message_print( channel, param, format, argp );
+    va_end( argp );
+}    
+
+void mprf( msg_channel_type channel, const char *format, ... )
 {
     va_list  argp;
     va_start( argp, format );
@@ -271,8 +388,14 @@ void mprf( const char *format, ... )
     va_end( argp );
 }
 
-void mpr(const char *inf, int channel, int param)
+void mpr(const char *inf, msg_channel_type channel, int param)
 {
+    if (!crawl_state.io_inited)
+    {
+        if (channel == MSGCH_WARN)
+            fprintf(stderr, "%s\n", inf);
+        return;
+    }
     char mbuf[400];
     size_t i = 0;
     const int stepsize = get_number_of_cols() - 1;
@@ -313,9 +436,43 @@ void mpr(const char *inf, int channel, int param)
     }
 }
 
+// mpr() an arbitrarily long list of strings without truncation or risk
+// of overflow.
+void mpr_comma_separated_list(const std::string prefix,
+                              const std::vector<std::string> list,
+                              const std::string &andc,
+                              const std::string &comma,
+                              const msg_channel_type channel,
+                              const int param)
+{
+    std::string out = prefix;
+    unsigned width  = get_number_of_cols() - 1;
+
+    for(int i = 0, size = list.size(); i < size; i++)
+    {
+        std::string new_str = list[i];
+
+        if (size > 0 && i < (size - 2))
+            new_str += comma;
+        else if (i == (size - 2))
+            new_str += andc;
+
+        if (out.length() + new_str.length() >= width)
+        {
+            mpr(out.c_str(), channel, param);
+            out = new_str;
+        }
+        else
+            out += new_str;
+    }
+    mpr(out.c_str(), channel, param);
+}
+
+
 // checks whether a given message contains patterns relevant for
 // notes, stop_running or sounds and handles these cases
-static void mpr_check_patterns(const std::string& message, int channel,
+static void mpr_check_patterns(const std::string& message,
+                               msg_channel_type channel,
                                int param)
 {
     for (unsigned i = 0; i < Options.note_messages.size(); ++i)
@@ -327,8 +484,14 @@ static void mpr_check_patterns(const std::string& message, int channel,
         }
     }
 
-    if (channel != MSGCH_DIAGNOSTICS && channel != MSGCH_EQUIPMENT)
-        interrupt_activity( AI_MESSAGE, channel_to_str(channel) + ":" + message );
+    // reusing travel_stop_message here
+    if (channel != MSGCH_DIAGNOSTICS && channel != MSGCH_EQUIPMENT
+        && channel != MSGCH_TALK && channel != MSGCH_TALK_VISUAL
+        && channel != MSGCH_SOUND)
+    {
+        interrupt_activity( AI_MESSAGE,
+                            channel_to_str(channel) + ":" + message );
+    }
 
     // Check messages for all forms of running now.
     if (you.running)
@@ -361,9 +524,9 @@ static void mpr_check_patterns(const std::string& message, int channel,
 
 // adds a given message to the message history
 static void mpr_store_messages(const std::string& message,
-                               int channel, int param)
+                               msg_channel_type channel, int param)
 {
-    const int num_lines = get_message_window_height();
+    const int num_lines = crawl_view.msgsz.y;
 
     // Prompt lines are presumably shown to / seen by the player accompanied
     // by a request for input, which should do the equivalent of a more(); to
@@ -378,7 +541,7 @@ static void mpr_store_messages(const std::string& message,
     textcolor(LIGHTGREY);
 
     // equipment lists just waste space in the message recall
-    if (channel != MSGCH_EQUIPMENT)
+    if (channel != MSGCH_EQUIPMENT && channel != MSGCH_EXAMINE_FILTER)
     {
         // Put the message into Store_Message, and move the '---' line forward
         Store_Message[ Next_Message ].text = message;
@@ -396,7 +559,8 @@ static bool need_prefix = false;
 // Does the work common to base_mpr and formatted_mpr.
 // Returns the default colour of the message, or MSGCOL_MUTED if
 // the message should be suppressed.
-static int prepare_message(const std::string& imsg, int channel, int param)
+static int prepare_message(const std::string& imsg, msg_channel_type channel,
+                           int param)
 {
     if (suppress_messages)
         return MSGCOL_MUTED;    
@@ -420,7 +584,7 @@ static int prepare_message(const std::string& imsg, int channel, int param)
     {
         mpr_check_patterns(imsg, channel, param);
         flush_input_buffer( FLUSH_ON_MESSAGE );
-        const int num_lines = get_message_window_height();
+        const int num_lines = crawl_view.msgsz.y;
     
         if (New_Message_Count == num_lines - 1)
             more();
@@ -428,13 +592,19 @@ static int prepare_message(const std::string& imsg, int channel, int param)
     return colour;
 }
 
-static void base_mpr(const char *inf, int channel, int param)
+static void base_mpr(const char *inf, msg_channel_type channel, int param)
 {
     const std::string imsg = inf;
     const int colour = prepare_message( imsg, channel, param );
     
     if ( colour == MSGCOL_MUTED )
         return;
+
+    if (silenced(you.x_pos, you.y_pos) &&
+        (channel == MSGCH_SOUND || channel == MSGCH_TALK))
+    {
+        return;
+    }
    
     if (need_prefix)
     {
@@ -460,7 +630,6 @@ static void mpr_formatted_output(formatted_string fs, int colour)
         need_prefix = false;
     }
 
-
     for ( unsigned i = 0; i < fs.ops.size(); ++i )
     {
         switch ( fs.ops[i].type )
@@ -471,7 +640,7 @@ static void mpr_formatted_output(formatted_string fs, int colour)
         case FSOP_TEXT:
             message_out(Message_Line, colour, fs.ops[i].text.c_str(), curcol,
                         false);
-            curcol += fs.ops[i].text.length();
+            curcol += multibyte_strlen(fs.ops[i].text);
             break;
         case FSOP_CURSOR:
             break;
@@ -483,9 +652,8 @@ static void mpr_formatted_output(formatted_string fs, int colour)
 // Line wrapping is not available here!
 // Note that the colour will be first set to the appropriate channel
 // colour before displaying the formatted_string.
-// XXX This code just reproduces base_mpr(). There must be a better
-// way to do this.
-void formatted_mpr(const formatted_string& fs, int channel, int param)
+void formatted_mpr(const formatted_string& fs, msg_channel_type channel,
+                   int param)
 {
     const std::string imsg = fs.tostring();
     const int colour = prepare_message(imsg, channel, param);
@@ -499,7 +667,8 @@ void formatted_mpr(const formatted_string& fs, int channel, int param)
 // output given string as formatted message, but check patterns
 // for string stripped of tags and store original tagged string
 // for message history
-void formatted_message_history(const std::string st, int channel, int param)
+void formatted_message_history(const std::string &st, msg_channel_type channel,
+                               int param)
 {
     if (suppress_messages)
         return;
@@ -514,7 +683,7 @@ void formatted_message_history(const std::string st, int channel, int param)
 
     flush_input_buffer( FLUSH_ON_MESSAGE );
 
-    const int num_lines = get_message_window_height();
+    const int num_lines = crawl_view.msgsz.y;
 
     if (New_Message_Count == num_lines - 1)
         more();
@@ -553,27 +722,35 @@ void mesclr( bool force )
 
 void more(void)
 {
-    char keypress = 0;
+#ifdef DEBUG_DIAGNOSTICS
+    if (you.running)
+    {
+        mesclr();
+        return;
+    }
+#endif
+    if (Options.show_more_prompt && !suppress_messages)
+    {
+        char keypress = 0;
 
-    if (Options.tutorial_left)
-        message_out(get_message_window_height() - 1,
-                    LIGHTGREY,
-                    "--more--                        "
-                    "Press Ctrl-P to reread old messages",
-                    2, false);
-    else
-        message_out(get_message_window_height() - 1,
-                    LIGHTGREY, "--more--", 2, false);
+        if (Options.tutorial_left)
+            message_out(crawl_view.msgsz.y - 1,
+                        LIGHTGREY,
+                        "--more--                        "
+                        "Press Ctrl-P to reread old messages",
+                        2, false);
+        else
+            message_out(crawl_view.msgsz.y - 1,
+                        LIGHTGREY, "--more--", 2, false);
 
-    do
-        keypress = getch();
-    while (keypress != ' ' && keypress != '\r' && keypress != '\n');
-
-    mesclr( !Options.delay_message_clear &&
-            Message_Line >= get_message_window_height() - 1 );
+        do
+            keypress = getch();
+        while (keypress != ' ' && keypress != '\r' && keypress != '\n');
+    }
+    mesclr(true);
 }                               // end more()
 
-static bool is_channel_dumpworthy(int channel)
+static bool is_channel_dumpworthy(msg_channel_type channel)
 {
     return (channel != MSGCH_EQUIPMENT
             && channel != MSGCH_DIAGNOSTICS
@@ -690,7 +867,7 @@ void replay_messages(void)
                         textcolor( colour );
                         gotoxy(curcol, wherey());
                         cprintf(fs.ops[j].text.c_str());
-                        curcol += fs.ops[j].text.length();
+                        curcol += multibyte_strlen(fs.ops[j].text);
                         break;
                     case FSOP_CURSOR:
                         break;
