@@ -3,7 +3,7 @@
  *  Summary:    Misc monster related functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2008-01-28T17:02:22.871440Z $
  *
  *  Change History (most recent first):
  *
@@ -882,7 +882,8 @@ void monster_teleport(struct monsters *monster, bool instan, bool silent)
         {
             if (!silent)
                 simple_monster_message(monster, " looks slightly unstable.");
-            monster->add_ench( mon_enchant(ENCH_TP, coinflip()? 3 : 4) );
+            monster->add_ench( mon_enchant(ENCH_TP, 0, KC_OTHER,
+                                           random_range(20, 30)) );
         }
 
         return;
@@ -894,9 +895,13 @@ void monster_teleport(struct monsters *monster, bool instan, bool silent)
         simple_monster_message(monster, " disappears!");
 
     const coord_def oldplace = monster->pos();
+
     // pick the monster up
     mgrd(oldplace) = NON_MONSTER;
 
+    if (mons_is_caught(monster))
+        mons_clear_trapping_net(monster);
+
     int newx, newy;
     while(true)
     {
@@ -942,9 +947,6 @@ void monster_teleport(struct monsters *monster, bool instan, bool silent)
     monster->check_redraw(oldplace);
     monster->apply_location_effects();
 
-    if (monster->has_ench(ENCH_HELD))
-        monster->del_ench(ENCH_HELD, true);
-
     // Teleporting mimics change form - if they reappear out of LOS, they are
     // no longer known.
     if (mons_is_mimic(monster->type))
@@ -1423,14 +1425,16 @@ void spore_goes_pop(struct monsters *monster)
         return;
     }
 
-    if (mons_near(monster))
+    bool nearby = mons_near(monster);
+    
+    if (nearby)
     {
-        viewwindow(1, false);
+        viewwindow(true, false);
         mpr(msg);
     }
 
-    explosion(beam);
-}                               // end spore_goes_pop()
+    explosion(beam, false, false, true, true, nearby);
+}
 
 bolt mons_spells( int spell_cast, int power )
 {
@@ -2294,3 +2298,15 @@ bool moth_incite_monsters(const monsters *mon)
 
     return (false);
 }
+
+void mons_clear_trapping_net(monsters *mons)
+{
+    if (!mons_is_caught(mons))
+        return;
+
+    const int net = get_trapping_net(mons->x, mons->y);
+    if (net != NON_ITEM)
+        remove_item_stationary(mitm[net]);
+
+    mons->del_ench(ENCH_HELD, true);
+}
