@@ -3,7 +3,7 @@
  * Summary:    Tracking monsters in transit between levels.
  * Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-15T20:10:20.648083Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-22T16:20:45.803473Z $
  */
 
 #ifndef MTRANSIT_H
@@ -31,9 +31,11 @@ typedef std::map<level_id, m_transit_list> monsters_in_transit;
 
 extern monsters_in_transit the_lost_ones;
 
+m_transit_list *get_transit_list(const level_id &where);
 void add_monster_to_transit(const level_id &dest, const monsters &m);
 
 // Places (some of the) monsters eligible to be placed on this level.
 void place_transiting_monsters();
+void place_followers();
 
 #endif
