@@ -3,7 +3,7 @@
  *  Summary:    Functions used when Bad Things happen to the player.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:58:46.756027Z $
  *
  *  Change History (most recent first):
  *
@@ -744,14 +744,6 @@ void ouch( int dam, int death_source, kill_method_type death_type,
         return;
     }
 
-    // assumed bug for high damage amounts
-    if (dam > 300)
-    {
-        mprf(MSGCH_DANGER,
-             "Potential bug: Unexpectedly high damage = %d", dam );
-        return;                 
-    }
-
     if (dam > -9000)            // that is, a "death" caused by hp loss {dlb}
     {
         switch (you.religion)
