@@ -3,7 +3,7 @@
  *  Summary:    Functions used to print player related info.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:29:24.300708Z $
  *
  *  Change History (most recent first):
  *
@@ -459,6 +459,12 @@ void print_stats(void)
             cprintf( "Holy " );
         }
 
+        if (you.duration[DUR_TELEPORT])
+        {
+            textcolor( LIGHTBLUE );
+            cprintf( "Tele " );
+        }        
+
         if (you.duration[DUR_DEFLECT_MISSILES])
         {
 
