@@ -3,7 +3,7 @@
  *  Summary:    Random and unrandom artefact functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:44:32.335576Z $
  *
  *  Change History (most recent first):
  *
@@ -581,7 +581,6 @@ static const char *rand_armour_names[] = {
     " of the Evening Star",
     " of the Seven Stars",
     " of the Seventy-Seven Stars",
-    " of the Seven Hundred Seventy-Seven Stars",
     " of Departure",
     " of Arrival",
     " of Change",
@@ -734,15 +733,15 @@ static const char *rand_armour_names[] = {
 };
 
 static const char *randart_weapon_appearance[] = {
-    "brightly glowing ", "runed ", "smoking ", "bloodstained ", "twisted ",
+    "brightly glowing ", "iridescent ", "smoking ", "bloodstained ", "twisted ",
     "shimmering ", "warped ", "crystal ", "jewelled ", "transparent ",
     "encrusted ", "pitted ", "slimy ", "polished ", "fine ", "crude ",
     "ancient ", "ichor-stained ", "faintly glowing ", "steaming ", "shiny "
 };
 
 static const char *randart_armour_appearance[] = {
-    "brightly glowing ", "runed ", "smoking ", "bloodstained ", "twisted ",
-    "shimmering ", "warped ", "heavily runed ", "jewelled ",
+    "brightly glowing ", "faintly humming ", "smoking ", "bloodstained ",
+    "twisted ", "shimmering ", "warped ", "heavily runed ", "jewelled ",
     "transparent ", "encrusted ", "pitted ", "slimy ", "polished ", "fine ",
     "crude ", "ancient ", "ichor-stained ", "faintly glowing ",
     "steaming ", "shiny ", "distressingly furry "
