@@ -3,7 +3,7 @@
  *  Summary:    Shop keeper functions.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-05-05T07:45:25.062795Z $
+ *  Modified for Crawl Reference by $Author: haranp $ on $Date: 2007-11-30T14:03:02.050008Z $
  *
  *  Change History (most recent first):
  *
@@ -25,9 +25,11 @@
 #endif
 
 #include "externs.h"
-
+#include "cio.h"
 #include "describe.h"
+#include "food.h"
 #include "invent.h"
+#include "it_use2.h"
 #include "items.h"
 #include "itemname.h"
 #include "itemprop.h"
@@ -40,14 +42,10 @@
 #include "stuff.h"
 #include "view.h"
 
-static char in_a_shop(char shoppy, id_arr id);
-static char more3(void);
-static void purchase( int shop, int item_got, int cost );
-static void shop_init_id(int i, id_fix_arr &shop_id);
-static void shop_print(const char *shoppy, char sh_line);
-static void shop_set_ident_type(int i, id_fix_arr &shop_id,
-                                unsigned char base_type, unsigned char sub_type);
-static void shop_uninit_id(int i, const id_fix_arr &shop_id);
+static void in_a_shop(int shopidx);
+static void more3();
+static void purchase( int shop, int item_got, int cost, bool id);
+static void shop_print(const char *shoppy, int sh_line);
 
 static std::string hyphenated_suffix(char prev, char last)
 {
@@ -91,7 +89,7 @@ static void list_shop_keys(const std::string &purchasable)
     gotoxy(1, numlines - 1);
 
     std::string pkeys = purchase_keys(purchasable);
-    if (pkeys.length())
+    if (!pkeys.empty())
         pkeys = "[" + pkeys + "] Buy Item";
 
     snprintf(buf, sizeof buf,
@@ -110,362 +108,243 @@ static void list_shop_keys(const std::string &purchasable)
     fs.display();
 }
 
-char in_a_shop( char shoppy, id_arr id )
+static std::vector<int> shop_stock(int shopidx)
 {
-    // easier to work with {dlb}
-    unsigned int greedy = env.shop[shoppy].greed;
-    cursor_control coff(false);
-
-    id_fix_arr shop_id;
-    FixedVector < int, 20 > shop_items;
-
-    char st_pass[ ITEMNAME_SIZE ] = "";
-    unsigned int gp_value = 0;
-    char i;
-    unsigned char ft;
-    std::string purchasable;
-
-    clrscr();
-    int itty = 0;
-
-    snprintf( info, INFO_SIZE, "Welcome to %s!", 
-             shop_name(env.shop[shoppy].x, env.shop[shoppy].y) );
-
-    ShopInfo &si = stashes.get_shop(env.shop[shoppy].x, env.shop[shoppy].y);
-
-    shop_print(info, 20);
-
-    more3();
+    std::vector<int> result;
     
-    activate_notes(false);      /* should do a better job here */
-    shop_init_id(shoppy, shop_id);
+    int itty = igrd[0][5 + shopidx];
 
-    /* *************************************
-    THINGS TO DO:
-      Allow inventory
-      Remove id change for antique shops
-      selling?
-    ************************************* */
-
-    save_id(id);
-
-  print_stock:
-    clrscr();
-    itty = igrd[0][5 + shoppy];
-
-    si.reset();
-
-    if (itty == NON_ITEM)
+    while ( itty != NON_ITEM )
     {
-        shop_print("I'm sorry, my shop is empty now.", 20);
-        more3();
-        goto goodbye;
-    }
-
-    for (i = 1; i < 20; i++)
-    {
-        shop_items[i - 1] = itty;
-
-        if (itty == NON_ITEM)   //mitm.link [itty] == NON_ITEM)
-        {
-            shop_items[i - 1] = NON_ITEM;
-            continue;
-        }
-
+        result.push_back( itty );
         itty = mitm[itty].link;
     }
+    return result;
+}
 
-    itty = igrd[0][5 + shoppy];
-
-    purchasable.clear();
-    for (i = 1; i < 18; i++)
+static int shop_item_value(const item_def& item, int greed, bool id)
+{
+    int result = (greed * item_value(item, id) / 10);
+    if ( you.duration[DUR_BARGAIN] ) // 20% discount
     {
-        const char c = i + 96;
+        result *= 8;
+        result /= 10;
+    }
+    
+    if (result < 1)
+        result = 1;
 
-        gotoxy(1, i);
-
-        gp_value = greedy * item_value( mitm[itty], id );
-        gp_value /= 10;
-        if (gp_value <= 1)
-            gp_value = 1;
+    return result;
+}
 
+static std::string shop_print_stock( const std::vector<int>& stock,
+                                     const shop_struct& shop )
+{
+    ShopInfo &si = stashes.get_shop(shop.x, shop.y);
+    const bool id = shoptype_identifies_stock(shop.type);
+    std::string purchasable;
+    for (unsigned int i = 0; i < stock.size(); ++i)
+    {
+        const int gp_value = shop_item_value(mitm[stock[i]], shop.greed, id);
         const bool can_afford = (you.gold >= gp_value);
-        textcolor( can_afford ? LIGHTGREEN : LIGHTRED );
 
+        gotoxy(1, i+1);
+        const char c = i + 'a';
         if (can_afford)
             purchasable += c;
 
+        textcolor( can_afford ? LIGHTGREEN : LIGHTRED );
         cprintf("%c - ", c);
 
-        textcolor((i % 2) ? WHITE : LIGHTGREY);
-
-        it_name(itty, DESC_NOCAP_A, st_pass);
-        cprintf("%s", st_pass);
-
-        std::string desc;
-        if (is_dumpable_artifact(mitm[itty], Options.verbose_dump))
-            desc = munge_description(get_item_description(mitm[itty], 
-                                                          Options.verbose_dump,
-                                                          true ));
-        si.add_item(mitm[itty], gp_value);
-
-        gotoxy(60, i);
-        cprintf("%5d gold", gp_value);
-        if (mitm[itty].link == NON_ITEM)
-            break;
-
-        itty = mitm[itty].link;
+        textcolor((i % 2) ? LIGHTGREY : WHITE);
+        cprintf("%-56s%5d gold",
+                mitm[stock[i]].name(DESC_NOCAP_A, false, id).c_str(),
+                gp_value);
+        si.add_item(mitm[stock[i]], gp_value);
     }
-
     textcolor(LIGHTGREY);
+    return purchasable;
+}
 
-    list_shop_keys(purchasable);
+static void in_a_shop( int shopidx )
+{
+    const shop_struct& shop = env.shop[shopidx];
 
-  purchase:
-    snprintf( info, INFO_SIZE, "You have %d gold piece%s.", you.gold,
-             (you.gold == 1) ? "" : "s" );
+    cursor_control coff(false);
 
-    textcolor(YELLOW);
-    shop_print(info, 19);
+    const std::string hello = "Welcome to " + shop_name(shop.x, shop.y) + "!";
+    shop_print(hello.c_str(), 20);
 
-    textcolor(CYAN);
-
-    snprintf(st_pass, sizeof st_pass, 
-            "What would you like to %s?",
-                purchasable.length()? "purchase" : "do");
-    shop_print(st_pass, 20);
-    textcolor(LIGHTGREY);
-
-    ft = get_ch();
-
-    if (ft == 'x' || ft == ESCAPE)
-        goto goodbye;
-
-    if (ft == '\\')
+    more3();    
+    
+    const bool id_stock = shoptype_identifies_stock(shop.type);
+ 
+    while ( true )
     {
-        shop_uninit_id(shoppy, shop_id);
-        check_item_knowledge();
-        shop_init_id(shoppy, shop_id);
+        stashes.get_shop(shop.x, shop.y).reset();
+        
+        std::vector<int> stock = shop_stock(shopidx);
 
-        goto print_stock;
-    }
-
-    if (ft == 'v')
-    {
-        textcolor(CYAN);
-        shop_print("Examine which item?", 20);
-        textcolor(LIGHTGREY);
-        ft = get_ch();
-
-        // wonder whether this should be recoded to permit uppercase, too? {dlb}
-        if (ft < 'a' || ft > 'z')
-            goto huh;
-
-        ft -= 'a';              // see above comment {dlb}
-
-        if (ft > 18)
-            goto huh;
-
-        if (shop_items[ft] == NON_ITEM)
+        clrscr();
+        if ( stock.empty() )
         {
-            shop_print("I'm sorry, you seem to be confused.", 20);
+            shop_print("I'm sorry, my shop is empty now.", 20);
             more3();
-            goto purchase;
+            return;
         }
+            
+        const std::string purchasable = shop_print_stock(stock, shop);
+        list_shop_keys(purchasable);
+            
+        snprintf( info, INFO_SIZE, "You have %d gold piece%s.", you.gold,
+                  (you.gold == 1) ? "" : "s" );
+        textcolor(YELLOW);
+        shop_print(info, 19);
 
-        describe_item( mitm[shop_items[ft]] );
+        snprintf( info, INFO_SIZE, "What would you like to %s?",
+                  purchasable.length()? "purchase" : "do");
+        textcolor(CYAN);
+        shop_print(info, 20);
+            
+        textcolor(LIGHTGREY);
 
-        goto print_stock;
-    }
-
-    if (ft == '?' || ft == '*')
-    {
-        shop_uninit_id(shoppy, shop_id);
-        invent(-1, false);
-        shop_init_id(shoppy, shop_id);
-        goto print_stock;
-    }
-
-    if (ft < 'a' || ft > 'z')   // see earlier comments re: uppercase {dlb}
-    {
-      huh:
-        shop_print("Huh?", 20);
-        more3();
-        goto purchase;
-    }
-
-    ft -= 'a';                  // see earlier comments re: uppercase {dlb}
-
-    if (ft > 18)
-        goto huh;
-
-    if (shop_items[ft] == NON_ITEM)
-    {
-        shop_print("I'm sorry, you seem to be confused.", 20);
-        more3();
-        goto purchase;
-    }
-
-    gp_value = greedy * item_value( mitm[shop_items[ft]], id ) / 10;
-
-    if (gp_value > you.gold)
-    {
-        shop_print("I'm sorry, you don't seem to have enough money.", 20);
-        more3();
-        goto purchase;
-    }
-    snprintf(info, INFO_SIZE, "Purchase %s (%d gold)? [y/n]",
-             item_name(mitm[shop_items[ft]], DESC_NOCAP_A), gp_value);
-    shop_print(info, 20);
-    if ( yesno(NULL, true, 'n', false, false, true) )
-    {        
-        shop_set_ident_type( shoppy, shop_id, mitm[shop_items[ft]].base_type,
-                             mitm[shop_items[ft]].sub_type );
-        purchase( shoppy, shop_items[ft], gp_value );
-    }
-
-    goto print_stock;
-
-  goodbye:
-    //clear_line();
-    shop_print("Goodbye!", 20);
-    more3();
-
-    shop_uninit_id( shoppy, shop_id );
-    activate_notes(true);
-    return 0;
-}
-
-void shop_init_id_type(int shoptype, id_fix_arr &shop_id)
-{
-    if (shoptype != SHOP_WEAPON_ANTIQUE
-        && shoptype != SHOP_ARMOUR_ANTIQUE
-        && shoptype != SHOP_GENERAL_ANTIQUE)
-    {
-        for (int j = 0; j < 50; j++)
+        int ft = get_ch();
+        
+        if ( ft == '\\' )
+            check_item_knowledge();
+        else if (ft == 'x' || ft == ESCAPE)
+            break;
+        else if (ft == 'v')
         {
-            shop_id[ IDTYPE_WANDS ][j] = get_ident_type(OBJ_WANDS, j);
-            set_ident_type(OBJ_WANDS, j, ID_KNOWN_TYPE);
+            textcolor(CYAN);
+            shop_print("Examine which item?", 20);
+            textcolor(LIGHTGREY);
 
-            shop_id[ IDTYPE_SCROLLS ][j] = get_ident_type(OBJ_SCROLLS, j);
-            set_ident_type(OBJ_SCROLLS, j, ID_KNOWN_TYPE);
+            bool is_ok = true;
 
-            shop_id[ IDTYPE_JEWELLERY ][j] = get_ident_type(OBJ_JEWELLERY, j);
-            set_ident_type(OBJ_JEWELLERY, j, ID_KNOWN_TYPE);
+            ft = get_ch();
+            if ( !isalpha(ft) )
+            {
+                is_ok = false;
+            }
+            else
+            {
+                ft = tolower(ft) - 'a';
+                if ( ft >= static_cast<int>(stock.size()) )
+                    is_ok = false;
+            }
 
-            shop_id[ IDTYPE_POTIONS ][j] = get_ident_type(OBJ_POTIONS, j);
-            set_ident_type(OBJ_POTIONS, j, ID_KNOWN_TYPE);
+            if ( !is_ok )
+            {
+                shop_print("Huh?", 20);
+                more3();
+                continue;
+            }
+
+            // A hack to make the description more useful.
+            // In theory, the user could kill the process at this
+            // point and end up with valid ID for the item.
+            // That's not very useful, though, because it doesn't set
+            // type-ID and once you can access the item (by buying it)
+            // you have its full ID anyway. Worst case, it won't get
+            // noted when you buy it.
+            item_def& item = mitm[stock[ft]];
+            const unsigned long old_flags = item.flags;
+            if ( id_stock )
+                item.flags |= (ISFLAG_IDENT_MASK | ISFLAG_NOTED_ID |
+                               ISFLAG_NOTED_GET);
+            describe_item(item);
+            if ( id_stock )
+                item.flags = old_flags;
         }
-    }
-}
-
-static void shop_init_id(int i, id_fix_arr &shop_id)
-{
-    shop_init_id_type( env.shop[i].type, shop_id );
-}
-
-void shop_uninit_id_type(int shoptype, const id_fix_arr &shop_id)
-{
-    if (shoptype != SHOP_WEAPON_ANTIQUE
-        && shoptype != SHOP_ARMOUR_ANTIQUE
-        && shoptype != SHOP_GENERAL_ANTIQUE)
-    {
-        for (int j = 0; j < 50; j++)
+        else if (ft == '?' || ft == '*')
+            invent(-1, false);
+        else if ( !isalpha(ft) )
         {
-            set_ident_type( OBJ_WANDS, j, 
-                    shop_id[ IDTYPE_WANDS ][j], true );
-            set_ident_type( OBJ_SCROLLS, j, 
-                    shop_id[ IDTYPE_SCROLLS ][j], true );
-            set_ident_type( OBJ_JEWELLERY, j, 
-                    shop_id[ IDTYPE_JEWELLERY ][j], true );
-            set_ident_type( OBJ_POTIONS, j, 
-                    shop_id[ IDTYPE_POTIONS ][j], true );
+            shop_print("Huh?", 20);
+            more3();
         }
-    }
-}
-
-static void shop_uninit_id(int i, const id_fix_arr &shop_id)
-{
-    shop_uninit_id_type(env.shop[i].type, shop_id);
-}
-
-void shop_set_ident_type( int i, id_fix_arr &shop_id,
-                          unsigned char base_type, unsigned char sub_type )
-{
-    if (env.shop[i].type != SHOP_WEAPON_ANTIQUE
-        && env.shop[i].type != SHOP_ARMOUR_ANTIQUE
-        && env.shop[i].type != SHOP_GENERAL_ANTIQUE)
-    {
-        switch (base_type)
+        else
         {
-        case OBJ_WANDS:
-            shop_id[ IDTYPE_WANDS ][sub_type] = ID_KNOWN_TYPE;
-            break;
-        case OBJ_SCROLLS:
-            shop_id[ IDTYPE_SCROLLS ][sub_type] = ID_KNOWN_TYPE;
-            break;
-        case OBJ_JEWELLERY:
-            shop_id[ IDTYPE_JEWELLERY ][sub_type] = ID_KNOWN_TYPE;
-            break;
-        case OBJ_POTIONS:
-            shop_id[ IDTYPE_POTIONS ][sub_type] = ID_KNOWN_TYPE;
-            break;
+            ft = tolower(ft) - 'a';
+            if (ft >= static_cast<int>(stock.size()) )
+            {
+                shop_print("No such item.", 20);
+                more3();
+                continue;
+            }
+
+            item_def& item = mitm[stock[ft]];
+            
+            const int gp_value = shop_item_value(item, shop.greed, id_stock);
+            if (gp_value > you.gold)
+            {
+                shop_print("I'm sorry, you don't seem to have enough money.",
+                           20);
+                more3();
+            }
+            else
+            {
+                snprintf(info, INFO_SIZE, "Purchase %s (%d gold)? [y/n]",
+                         item.name(DESC_NOCAP_A).c_str(), gp_value);
+                shop_print(info, 20);
+
+                if ( yesno(NULL, true, 'n', false, false, true) )
+                    purchase( shopidx, stock[ft], gp_value, id_stock );
+            }
         }
     }
 }
 
-void shop_print( const char *shoppy, char sh_lines )
+bool shoptype_identifies_stock(shop_type type)
+{
+    return
+        type != SHOP_WEAPON_ANTIQUE &&
+        type != SHOP_ARMOUR_ANTIQUE &&
+        type != SHOP_GENERAL_ANTIQUE;
+}
+
+static void shop_print( const char *shoppy, int sh_lines )
 {
     gotoxy(1, sh_lines);
-
     cprintf("%s", shoppy);
-
-    for (int i = strlen(shoppy); i < 80; i++)
-        cprintf(" ");
+    clear_to_end_of_line();
 }
 
-char more3(void)
+static void more3()
 {
-    char keyin = 0;
-
     gotoxy(70, 20);
     cprintf("-more-");
-    keyin = getch();
-    if (keyin == 0)
-        getch();
-    //clear_line();
-    return keyin;
+    get_ch();
+    return;
 }
 
-static void purchase( int shop, int item_got, int cost )
+static void purchase( int shop, int item_got, int cost, bool id )
 {
     you.gold -= cost;
 
-    origin_purchased(mitm[item_got]);
+    item_def& item = mitm[item_got];
 
-    if ( fully_identified(mitm[item_got]) &&
-         is_interesting_item(mitm[item_got]) ) {
+    origin_purchased(item);
 
-        activate_notes(true);
-
-        char buf[ITEMNAME_SIZE];
-        char buf2[ITEMNAME_SIZE];
-        item_name( mitm[item_got], DESC_NOCAP_A, buf );
-        strcpy(buf2, origin_desc(mitm[item_got]).c_str());
-        take_note(Note(NOTE_ID_ITEM, 0, 0, buf, buf2));
-
-        activate_notes(false);
+    if ( id )
+    {
+        // Identify the item and its type.
+        // This also takes the ID note if necessary.
+        set_ident_type(item.base_type, item.sub_type, ID_KNOWN_TYPE);
+        set_ident_flags(item, ISFLAG_IDENT_MASK);
     }
 
-    int num = move_item_to_player( item_got, mitm[item_got].quantity, true );
+    const int quant = item.quantity;
+    // note that item will be invalidated if num == item.quantity
+    const int num = move_item_to_player( item_got, item.quantity, true );
 
     // Shopkeepers will now place goods you can't carry outside the shop.
-    if (num < mitm[item_got].quantity)
+    if (num < quant)
     {
         snprintf( info, INFO_SIZE, "I'll put %s outside for you.",
-                 (mitm[item_got].quantity == 1) ? "it" :
-                 (num > 0)                      ? "the rest" 
-                                                : "these" );
+                 (quant == 1) ? "it" :
+                 (num > 0) ? "the rest" : "these" );
 
         shop_print( info, 20 );
         more3();
@@ -523,9 +402,9 @@ int randart_value( const item_def &item )
     if (prop[ RAP_ELECTRICITY ])
         ret += 30;
 
-    // magic resistance is from 20-120
+    // magic resistance is from 35-100
     if (prop[ RAP_MAGIC ])
-        ret += 5 + prop[ RAP_MAGIC ] / 10;
+        ret += 5 + prop[ RAP_MAGIC ] / 15;
 
     if (prop[ RAP_EYESIGHT ])
         ret += 10;
@@ -575,7 +454,7 @@ int randart_value( const item_def &item )
     return ((ret > 0) ? ret : 0);
 }
 
-unsigned int item_value( item_def item, id_arr id, bool ident )
+unsigned int item_value( item_def item, bool ident )
 {
     // Note that we pass item in by value, since we want a local
     // copy to mangle as necessary... maybe that should be fixed,
@@ -1135,8 +1014,7 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
             else
                 valued += 50;
         }
-        else if (item_type_known(item) 
-                && get_equip_desc(item) != 0)
+        else if (item_type_known(item) && get_equip_desc(item) != 0)
         {
             valued += 20;
         }
@@ -1149,7 +1027,7 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
         break;
 
     case OBJ_WANDS:
-        if (!id[ IDTYPE_WANDS ][item.sub_type])
+        if ( !item_type_known(item) )
             valued += 200;
         else
         {
@@ -1218,7 +1096,7 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
         break;
 
     case OBJ_POTIONS:
-        if (!id[3][item.sub_type])
+        if ( !item_type_known(item) )
             valued += 9;
         else
         {
@@ -1238,6 +1116,9 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
             case POT_MAGIC:
                 valued += 120;
                 break;
+            case POT_RESISTANCE:
+                valued += 70;
+                break;
             case POT_INVISIBILITY:
                 valued += 55;
                 break;
@@ -1253,6 +1134,7 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
             case POT_SPEED:
                 valued += 25;
                 break;
+            case POT_BLOOD:
             case POT_HEALING:
             case POT_LEVITATION:
                 valued += 20;
@@ -1329,9 +1211,10 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
         break;
 
     case OBJ_SCROLLS:
-        if (!id[1][item.sub_type])
+        if ( !item_type_known(item) )
             valued += 10;
         else
+        {
             switch (item.sub_type)
             {
             case SCR_ACQUIREMENT:
@@ -1384,16 +1267,16 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
                 valued++;
                 break;
             }
+        }
         break;
 
     case OBJ_JEWELLERY:
-        if (!id[2][item.sub_type])
-            valued += 50;
-
         if (item_cursed( item ))
             valued -= 10;
 
-        if (id[2][item.sub_type] > 0)
+        if ( !item_type_known(item) )
+            valued += 50;
+        else
         {
             if (item_ident( item, ISFLAG_KNOW_PLUSES ) 
                 && (item.sub_type == RING_PROTECTION
@@ -1485,17 +1368,12 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
 
             if (is_random_artefact(item))
             {
-                if (item_type_known(item))
-                {
-                    if (valued < 0)
-                        valued = randart_value( item ) - 5;
-                    else
-                        valued += randart_value( item );
-                }
+                // in this branch we're guaranteed to know
+                // the item type!
+                if (valued < 0)
+                    valued = randart_value( item ) - 5;
                 else
-                {
-                    valued += 50;
-                }
+                    valued += randart_value( item );
             }
 
             valued *= 7;
@@ -1546,20 +1424,16 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
             case MISC_BOTTLED_EFREET:
                 valued += 350;
                 break;
-            case MISC_DECK_OF_TRICKS:
-                valued += 100;
-                break;
             default:
                 valued += 400;
             }
         }
         break;
 
-    //case 10: break;
-
     case OBJ_BOOKS:
-        valued = 150 + (item_type_known(item) 
-                                    ? book_rarity(item.sub_type) * 50 : 0);
+        valued = 150;
+        if (item_type_known(item))
+            valued += book_rarity(item.sub_type) * 50;
         break;
 
     case OBJ_STAVES:
@@ -1583,6 +1457,8 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
     case OBJ_ORBS:
         valued = 250000;
         break;
+    default:
+        break;
     }                           // end switch
 
     if (valued < 1)
@@ -1593,38 +1469,26 @@ unsigned int item_value( item_def item, id_arr id, bool ident )
     return (valued);
 }                               // end item_value()
 
-void shop(void)
+void shop()
 {
-    unsigned char i = 0;
+    int i;
 
     for (i = 0; i < MAX_SHOPS; i++)
-    {
         if (env.shop[i].x == you.x_pos && env.shop[i].y == you.y_pos)
             break;
-    }
 
     if (i == MAX_SHOPS)
     {
-        mpr("Help! Non-existent shop.");
+        mpr("Help! Non-existent shop.", MSGCH_DANGER);
         return;
     }
 
-    id_arr identy;
-
-    save_id(identy);
-
-    {
-        shopping_hup_protect shp;
-        in_a_shop(i, identy);
-    }
-    
-    you.redraw_gold = 1;
+    in_a_shop(i);
     burden_change();
-
     redraw_screen();
 }                               // end shop()
 
-const shop_struct *get_shop(int sx, int sy)
+shop_struct *get_shop(int sx, int sy)
 {
     if (grd[sx][sy] != DNGN_ENTER_SHOP)
         return (NULL);
@@ -1642,9 +1506,16 @@ const shop_struct *get_shop(int sx, int sy)
     return (NULL);
 }
 
-const char *shop_name(int sx, int sy)
+std::string shop_name(int sx, int sy, bool add_stop)
+{
+    std::string name(shop_name(sx, sy));
+    if (add_stop)
+        name += ".";
+    return (name);
+}
+
+std::string shop_name(int sx, int sy)
 {
-    static char sh_name[80];
     const shop_struct *cshop = get_shop(sx, sy);
 
     // paranoia
@@ -1659,43 +1530,40 @@ const char *shop_name(int sx, int sy)
 
     int shop_type = cshop->type;
 
-    char st_p[ITEMNAME_SIZE];
-
     unsigned long seed = static_cast<unsigned long>( cshop->keeper_name[0] )
-                    | (static_cast<unsigned long>( cshop->keeper_name[1] ) << 8)
-                    | (static_cast<unsigned long>( cshop->keeper_name[1] ) << 16);
-    make_name( seed, false, st_p );
+        | (static_cast<unsigned long>( cshop->keeper_name[1] ) << 8)
+        | (static_cast<unsigned long>( cshop->keeper_name[1] ) << 16);
 
-    strcpy(sh_name, st_p);
-    strcat(sh_name, "'s ");
+    std::string sh_name = make_name(seed, false);
+    sh_name += "'s ";
 
     if (shop_type == SHOP_WEAPON_ANTIQUE || shop_type == SHOP_ARMOUR_ANTIQUE)
-        strcat( sh_name, "Antique " );
-
-    strcat(sh_name, (shop_type == SHOP_WEAPON
-                     || shop_type == SHOP_WEAPON_ANTIQUE) ? "Weapon" :
-                    (shop_type == SHOP_ARMOUR
-                     || shop_type == SHOP_ARMOUR_ANTIQUE) ? "Armour" :
-
-                    (shop_type == SHOP_JEWELLERY)         ? "Jewellery" :
-                    (shop_type == SHOP_WAND)              ? "Magical Wand" :
-                    (shop_type == SHOP_BOOK)              ? "Book" :
-                    (shop_type == SHOP_FOOD)              ? "Food" :
-                    (shop_type == SHOP_SCROLL)            ? "Magic Scroll" :
-                    (shop_type == SHOP_GENERAL_ANTIQUE) ? "Assorted Antiques" :
-                    (shop_type == SHOP_DISTILLERY)        ? "Distillery" :
-                    (shop_type == SHOP_GENERAL)           ? "General Store"
-                                                          : "Bug");
+        sh_name += "Antique ";
 
+    sh_name +=
+        (shop_type == SHOP_WEAPON
+         || shop_type == SHOP_WEAPON_ANTIQUE) ? "Weapon" :
+        (shop_type == SHOP_ARMOUR
+         || shop_type == SHOP_ARMOUR_ANTIQUE) ? "Armour" :
+        
+        (shop_type == SHOP_JEWELLERY)         ? "Jewellery" :
+        (shop_type == SHOP_WAND)              ? "Magical Wand" :
+        (shop_type == SHOP_BOOK)              ? "Book" :
+        (shop_type == SHOP_FOOD)              ? "Food" :
+        (shop_type == SHOP_SCROLL)            ? "Magic Scroll" :
+        (shop_type == SHOP_GENERAL_ANTIQUE)   ? "Assorted Antiques" :
+        (shop_type == SHOP_DISTILLERY)        ? "Distillery" :
+        (shop_type == SHOP_GENERAL)           ? "General Store"
+                                              : "Bug";
 
     if (shop_type != SHOP_GENERAL
         && shop_type != SHOP_GENERAL_ANTIQUE && shop_type != SHOP_DISTILLERY)
     {
         int temp = sx + sy % 4;
-        strcat( sh_name, (temp == 0) ? " Shoppe" :
-                         (temp == 1) ? " Boutique" :
-                         (temp == 2) ? " Emporium"
-                                     : " Shop" );
+        sh_name += (temp == 0) ? " Shoppe" :
+                   (temp == 1) ? " Boutique" :
+                   (temp == 2) ? " Emporium"
+                               : " Shop";
     }
 
     return (sh_name);
