@@ -3,7 +3,7 @@
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T15:09:45.680166Z $
  *
  *  Change History (most recent first):
  *
@@ -218,7 +218,7 @@ int cast_fire_storm(int powc, bolt &beam)
     beam.hit = 20 + powc / 10;
     beam.damage = calc_dice( 9, 20 + powc );
 
-    explosion( beam );
+    explosion( beam, false, false, true, true, false );
     mpr("A raging storm of fire appears!");
 
     viewwindow(1, false);
@@ -770,14 +770,6 @@ void extension(int pow)
     if (you.duration[DUR_SLOW])
         potion_effect(POT_SLOWING, pow);
 
-#if 0
-    if (you.duration[DUR_PARALYSIS])
-        potion_effect(POT_PARALYSIS, pow);  // how did you cast extension?
-
-    if (you.duration[DUR_CONF])
-        potion_effect(POT_CONFUSION, pow);  // how did you cast extension?
-#endif
-
     if (you.duration[DUR_MIGHT])
     {
         potion_effect(POT_MIGHT, pow);
