@@ -3,7 +3,7 @@
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-02T13:23:51.271663Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-12-03T12:42:36.774780Z $
  *
  *  Change History (most recent first):
  *
@@ -109,10 +109,11 @@ unsigned char detect_items( int pow )
     {
         for (int j = you.y_pos - map_radius; j < you.y_pos + map_radius; j++)
         {
-            if (i < 5 || j < 5 || i > (GXM - 5) || j > (GYM - 5))
+            if (!in_bounds(i, j))
                 continue;
 
-            if (igrd[i][j] != NON_ITEM)
+            if (igrd[i][j] != NON_ITEM
+                && (!get_envmap_obj(i, j) || !is_envmap_item(i, j)))
             {
                 set_envmap_obj(i, j, DNGN_ITEM_DETECTED);
                 set_envmap_detected_item(i, j);
