@@ -3,7 +3,7 @@
  *  Summary:    Implementations of some additional spells.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dolorous $ on $Date: 2007-12-21T04:33:42.219290Z $
  *
  *  Change History (most recent first):
  *
@@ -857,7 +857,9 @@ void cast_poison_ammo(void)
         || you.inv[ammo].base_type != OBJ_MISSILES
         || get_ammo_brand( you.inv[ammo] ) != SPMSL_NORMAL
         || you.inv[ammo].sub_type == MI_STONE
-        || you.inv[ammo].sub_type == MI_LARGE_ROCK)
+        || you.inv[ammo].sub_type == MI_SLING_BULLET
+        || you.inv[ammo].sub_type == MI_LARGE_ROCK
+        || you.inv[ammo].sub_type == MI_THROWING_NET)
     {
         canned_msg(MSG_NOTHING_HAPPENS);
         return;
