@@ -4,7 +4,7 @@
  *              other neglected areas of Crawl magic ;^)
  *  Written by: Copyleft Josh Fishman 1999-2000, All Rights Preserved
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T14:58:46.756027Z $
  *
  *  Change History (most recent first):
  *
@@ -1616,10 +1616,11 @@ static int passwall(int x, int y, int pow, int garbage)
 {
     UNUSED( garbage );
 
-    char dx, dy, nx = x, ny = y;
+    int dx, dy, nx = x, ny = y;
     int howdeep = 0;
     bool done = false;
     int shallow = 1 + (you.skills[SK_EARTH_MAGIC] / 8);
+    bool non_rock_barriers = false;
 
     // allow statues as entry points?
     if (grd[x][y] != DNGN_ROCK_WALL)
@@ -1637,9 +1638,7 @@ static int passwall(int x, int y, int pow, int garbage)
 
     while (!done)
     {
-        // I'm trying to figure proper borders out {dlb}
-        // FIXME: dungeon border?
-        if (nx > (GXM - 1) || ny > (GYM - 1) || nx < 2 || ny < 2)
+        if (!in_bounds(nx, ny))
         {
             mpr("You sense an overwhelming volume of rock.");
             return 0;
@@ -1648,8 +1647,11 @@ static int passwall(int x, int y, int pow, int garbage)
         switch (grd[nx][ny])
         {
         default:
+            if (grid_is_solid(grd[nx][ny]))
+                non_rock_barriers = true;
             done = true;
             break;
+            
         case DNGN_ROCK_WALL:
         case DNGN_ORCISH_IDOL:
         case DNGN_GRANITE_STATUE:
@@ -1663,13 +1665,14 @@ static int passwall(int x, int y, int pow, int garbage)
 
     int range = shallow + random2(pow) / 25;
 
-    if (howdeep > shallow)
+    if (howdeep > shallow || non_rock_barriers)
     {
-        mpr("This rock feels deep.");
+        mprf("This rock feels %sdeep.",
+             non_rock_barriers || (howdeep > range)? "extremely " : "");
 
         if (yesno("Try anyway?"))
         {
-            if (howdeep > range)
+            if (howdeep > range || non_rock_barriers)
             {
                 ouch(1 + you.hp, 0, KILLED_BY_PETRIFICATION);
                 //jmf: not return; if wizard, successful transport is option
