@@ -3,7 +3,7 @@
  *  Summary:    Spellbook/Staff contents array and management functions
  *  Written by: Josh Fishman
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T13:43:41.265643Z $
  *
  *  Change History (most recent first):
  *
@@ -750,10 +750,10 @@ int spellbook_contents( item_def &book, read_book_action_type action,
         int colour = DARKGREY;
         if (action == RBOOK_USE_STAFF)
         {
-            if (you.experience_level >= level_diff 
-                && (book.base_type == OBJ_BOOKS?
-                        you.magic_points >= level_diff
-                      : book.plus >= level_diff * ROD_CHARGE_MULT))
+            if (book.base_type == OBJ_BOOKS?
+                you.experience_level >= level_diff
+                && you.magic_points >= level_diff
+                : book.plus >= level_diff * ROD_CHARGE_MULT)
             {
                 colour = LIGHTGREY;
             }
@@ -1496,12 +1496,6 @@ int staff_spell( int staff )
         return (-1);
     }
 
-    if (you.experience_level < diff)
-    {
-        mprf("You need to be at least level %d to use that.", diff);
-        return (-1);
-    }
-
     const int flags = get_spell_flags(spell);
     // Labyrinths block divinations.
     if (you.level_type == LEVEL_LABYRINTH
