@@ -3,7 +3,7 @@
  *  Summary:    Classes tracking player stashes
  *  Written by: Darshan Shaligram
  *
- *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T16:56:51.330636Z $
  */
 
 #include "AppHdr.h"
@@ -24,6 +24,7 @@
 #include "menu.h"
 #include "message.h"
 #include "misc.h"
+#include "notes.h"
 #include "overmap.h"
 #include "place.h"
 #include "shopping.h"
@@ -563,6 +564,7 @@ void Stash::write(std::ostream &os,
                   bool identify) 
     const
 {
+    activate_notes(false);
     if (!enabled || (items.size() == 0 && verified)) return;
     os << "(" << ((int) x - refx) << ", " << ((int) y - refy)
        << (place.length()? ", " + place : "")
@@ -616,6 +618,7 @@ void Stash::write(std::ostream &os,
 
     if (items.size() <= 1 && !verified)
         os << "  (unseen)" << std::endl;
+    activate_notes(true);
 }
 
 void Stash::save(FILE *file) const
@@ -871,6 +874,7 @@ bool ShopInfo::matches_search(const std::string &prefix,
 
 void ShopInfo::write(std::ostream &os, bool identify) const
 {
+    activate_notes(false);
     os << "[Shop] " << name << std::endl;
     if (items.size() > 0)
     {
@@ -891,6 +895,7 @@ void ShopInfo::write(std::ostream &os, bool identify) const
         os << "  (Shop is empty)" << std::endl;
     else
         os << "  (Shop contents are unknown)" << std::endl;
+    activate_notes(true);
 }
 
 void ShopInfo::save(FILE *file) const
@@ -1384,7 +1389,10 @@ void StashTracker::show_stash_search_prompt()
     if (!prompt_qual.empty())
         prompt_qual = " [" + prompt_qual + "]";
 
-    mprf(MSGCH_PROMPT, "Search for what%s?\n", prompt_qual.c_str());
+    mprf(MSGCH_PROMPT, "Search for what%s?", prompt_qual.c_str());
+    // Push the cursor down to the next line. Newline on the prompt will not
+    // do the trick on DOS.
+    mpr("", MSGCH_PROMPT);    
 }
 
 void StashTracker::search_stashes()
