@@ -3,7 +3,7 @@
  *  Summary:    Misc stuff.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-11-24T17:24:21.713648Z $
+ *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2008-01-28T16:15:56.136594Z $
  *
  *  Change History (most recent first):
  *
@@ -14,6 +14,7 @@
  */
 
 #include "AppHdr.h"
+#include "cio.h"
 #include "database.h"
 #include "direct.h"
 #include "message.h"
@@ -633,6 +634,11 @@ int div_rand_round( int num, int den )
     return (num / den + (random2(den) < num % den));
 }
 
+int div_round_up( int num, int den )
+{
+    return (num / den + (num % den != 0));
+}
+
 bool one_chance_in(int a_million)
 {
     return (random2(a_million) == 0);
@@ -764,7 +770,7 @@ int yesnoquit( const char* str, bool safe, int safeanswer, bool clear_after )
 
         int tmp = getchm(KC_CONFIRM);
 
-        if ( tmp == ESCAPE || tmp == 'q' || tmp == 'Q' )
+        if ( tmp == CK_ESCAPE || tmp == 'q' || tmp == 'Q' )
             return -1;
         
         if ((tmp == ' ' || tmp == '\r' || tmp == '\n') && safeanswer)
@@ -785,7 +791,7 @@ int yesnoquit( const char* str, bool safe, int safeanswer, bool clear_after )
         else if (tmp == 'Y')
             return 1;
         else
-            mpr("[Y]es or [N]o only, please.");
+            mpr("[Y]es, [N]o or [Q]uit only, please.");
     }
 }    
 
