@@ -3,7 +3,7 @@
  *  Summary:    Misc function related to player transformations.
  *  Written by: Linley Henzell
  *
- *  Modified for Crawl Reference by $Author: dshaligram $ on $Date: 2007-03-04T11:05:55.333232Z $
+ *  Modified for Crawl Reference by $Author: ennewalker $ on $Date: 2007-11-23T02:25:44.246247Z $
  *
  *  Change History (most recent first):
  *
@@ -20,64 +20,78 @@
 
 #include "externs.h"
 
+#include "delay.h"
 #include "it_use2.h"
-#include "itemname.h"
+#include "itemprop.h"
 #include "items.h"
 #include "misc.h"
+#include "output.h"
 #include "player.h"
 #include "skills2.h"
 #include "stuff.h"
+#include "traps.h"
 
 void drop_everything(void);
 void extra_hp(int amount_extra);
 
-bool remove_equipment(FixedVector < char, 8 > &remove_stuff)
+bool remove_equipment(std::set<equipment_type> removed)
 {
-    char str_pass[ ITEMNAME_SIZE ];
-
     // if we're removing body armour, the cloak will come off as well -- bwr
-    if (remove_stuff[EQ_BODY_ARMOUR] == 1 && you.equip[EQ_BODY_ARMOUR] != -1)
-        remove_stuff[EQ_CLOAK] = 1;
+    if ( removed.find(EQ_BODY_ARMOUR) != removed.end() &&
+         you.equip[EQ_BODY_ARMOUR] != -1 )
+        removed.insert(EQ_CLOAK);
 
     // if we're removing gloves, the weapon will come off as well -- bwr
-    if (remove_stuff[EQ_GLOVES] == 1 && you.equip[EQ_GLOVES] != -1)
-        remove_stuff[EQ_WEAPON] = 1;
+    // but this makes Statue Form odd -- haranp
+    if ( removed.find(EQ_GLOVES) != removed.end() &&
+         you.equip[EQ_GLOVES] != -1 )
+        removed.insert(EQ_WEAPON);
 
-    if (remove_stuff[EQ_WEAPON] == 1 && you.equip[EQ_WEAPON] != -1)
+    if ( removed.find(EQ_WEAPON) != removed.end() &&
+         you.equip[EQ_WEAPON] != -1)
     {
-        unwield_item(you.equip[EQ_WEAPON]);
-        you.equip[EQ_WEAPON] = -1;
-        mpr("You are empty-handed.");
+        unwield_item();
+        canned_msg(MSG_EMPTY_HANDED);
         you.wield_change = true;
     }
 
-    for (int i = EQ_CLOAK; i < EQ_LEFT_RING; i++)
+    // Remove items in order (std::set is a sorted container)
+    std::set<equipment_type>::const_iterator iter;
+    for ( iter = removed.begin(); iter != removed.end(); ++iter )
     {
-        if (remove_stuff[i] == 0 || you.equip[i] == -1)
+        const equipment_type e = *iter;
+        if ( e == EQ_WEAPON || you.equip[e] == -1 )
             continue;
 
-        in_name( you.equip[i], DESC_CAP_YOUR, str_pass );
+        mprf("%s falls away.",
+             you.inv[you.equip[e]].name(DESC_CAP_YOUR).c_str());
 
-        snprintf( info, INFO_SIZE, "%s falls away.", str_pass );
-        mpr(info);
-
-        unwear_armour( you.equip[i] );
-        you.equip[i] = -1;
+        unwear_armour( you.equip[e] );
+        you.equip[e] = -1;
     }
 
     return true;
 }                               // end remove_equipment()
 
+bool remove_one_equip(equipment_type eq)
+{
+    std::set<equipment_type> r;
+    r.insert(eq);
+    return remove_equipment(r);
+}
+
 // Returns true if any piece of equipment that has to be removed is cursed.
 // Useful for keeping low level transformations from being too useful.
-static bool check_for_cursed_equipment( FixedVector < char, 8 > &remove_stuff )
+static bool check_for_cursed_equipment(const std::set<equipment_type> &remove)
 {
-    for (int i = EQ_WEAPON; i < EQ_LEFT_RING; i++)
+    std::set<equipment_type>::const_iterator iter;
+    for (iter = remove.begin(); iter != remove.end(); ++iter )
     {
-        if (remove_stuff[i] == 0 || you.equip[i] == -1)
+        equipment_type e = *iter;
+        if ( you.equip[e] == -1 )
             continue;
 
-        if (item_cursed( you.inv[ you.equip[i] ] ))
+        if (item_cursed( you.inv[ you.equip[e] ] ))
         {
             mpr( "Your cursed equipment won't allow you to complete the "
                  "transformation." );
@@ -101,6 +115,7 @@ size_type player::transform_size(int psize) const
     switch (transform)
     {
     case TRAN_SPIDER:
+    case TRAN_BAT:
         return SIZE_TINY;
     case TRAN_ICE_BEAST:
         return SIZE_LARGE;
@@ -114,7 +129,7 @@ size_type player::transform_size(int psize) const
     }
 }
 
-bool transform(int pow, char which_trans)
+bool transform(int pow, transformation_type which_trans)
 {
     if (you.species == SP_MERFOLK && player_is_swimming()
         && which_trans != TRAN_DRAGON)
@@ -152,7 +167,7 @@ bool transform(int pow, char which_trans)
     if (you.attribute[ATTR_TRANSFORMATION] != TRAN_NONE)
         untransform();
 
-    if (you.is_undead)
+    if (you.is_undead && (you.species != SP_VAMPIRE || which_trans != TRAN_BAT))
     {
         mpr("Your unliving flesh cannot be transformed in this way.");
         return (false);
@@ -161,13 +176,17 @@ bool transform(int pow, char which_trans)
     //jmf: silently discard this enchantment
     you.duration[DUR_STONESKIN] = 0;
 
-    FixedVector < char, 8 > rem_stuff;
+    // We drop everything except jewellery by default
+    equipment_type default_rem[] = {
+        EQ_WEAPON, EQ_CLOAK, EQ_HELMET, EQ_GLOVES, EQ_BOOTS,
+        EQ_SHIELD, EQ_BODY_ARMOUR
+    };
 
-    for (int i = EQ_WEAPON; i < EQ_RIGHT_RING; i++)
-        rem_stuff[i] = 1;
+    std::set<equipment_type> rem_stuff(default_rem,
+                                       default_rem + ARRAYSIZE(default_rem));
 
-    you.redraw_evasion = 1;
-    you.redraw_armour_class = 1;
+    you.redraw_evasion = true;
+    you.redraw_armour_class = true;
     you.wield_change = true;
 
     /* Remember, it can still fail in the switch below... */
@@ -178,7 +197,7 @@ bool transform(int pow, char which_trans)
             return (false);
 
         mpr("You turn into a venomous arachnid creature.");
-        remove_equipment( rem_stuff );
+        remove_equipment(rem_stuff);
 
         you.attribute[ATTR_TRANSFORMATION] = TRAN_SPIDER;
         you.duration[DUR_TRANSFORMATION] = 10 + random2(pow) + random2(pow);
@@ -192,10 +211,36 @@ bool transform(int pow, char which_trans)
         you.colour = BROWN;
         return (true);
 
-    case TRAN_ICE_BEAST:  // also AC +3, cold +3, fire -1, pois +1 
-        mpr( "You turn into a creature of crystalline ice." );
+    case TRAN_BAT:
+        // high ev, low ac, high speed
+        if (check_for_cursed_equipment( rem_stuff ))
+            return false;
+    
+        mprf("You turn into a %sbat.",
+             you.species == SP_VAMPIRE ? "vampire " : "");
 
-        rem_stuff[ EQ_CLOAK ] = 0;       
+        remove_equipment( rem_stuff );
+
+        you.attribute[ATTR_TRANSFORMATION] = TRAN_BAT;
+        you.duration[DUR_TRANSFORMATION] = 20 + random2(pow) + random2(pow);
+
+        if (you.duration[DUR_TRANSFORMATION] > 100)
+            you.duration[DUR_TRANSFORMATION] = 100;
+
+       modify_stat( STAT_DEXTERITY, 5, true );
+       modify_stat( STAT_STRENGTH, -5, true );
+   
+       you.symbol = 'b';
+       you.colour = DARKGREY;
+       return (true);
+        
+    case TRAN_ICE_BEAST:  // also AC +3, cold +3, fire -1, pois +1 
+        rem_stuff.erase(EQ_CLOAK);
+
+        if (check_for_cursed_equipment( rem_stuff ))
+            return false;
+            
+        mpr( "You turn into a creature of crystalline ice." );
 
         remove_equipment( rem_stuff );
 
@@ -215,10 +260,10 @@ bool transform(int pow, char which_trans)
         return (true);
 
     case TRAN_BLADE_HANDS:
-        rem_stuff[EQ_CLOAK] = 0;
-        rem_stuff[EQ_HELMET] = 0;
-        rem_stuff[EQ_BOOTS] = 0;
-        rem_stuff[EQ_BODY_ARMOUR] = 0;
+        rem_stuff.erase(EQ_CLOAK);
+        rem_stuff.erase(EQ_HELMET);
+        rem_stuff.erase(EQ_BOOTS);
+        rem_stuff.erase(EQ_BODY_ARMOUR);
 
         if (check_for_cursed_equipment( rem_stuff ))
             return (false);
@@ -234,6 +279,14 @@ bool transform(int pow, char which_trans)
         return (true);
 
     case TRAN_STATUE: // also AC +20, ev -5, elec +1, pois +1, neg +1, slow
+        rem_stuff.erase(EQ_WEAPON); // can still hold a weapon
+        rem_stuff.erase(EQ_CLOAK);
+        rem_stuff.erase(EQ_HELMET);
+        rem_stuff.erase(EQ_BOOTS);
+
+        if (check_for_cursed_equipment( rem_stuff ))
+            return false;
+
         if (you.species == SP_GNOME && coinflip())
             mpr( "Look, a garden gnome.  How cute!" );
         else if (player_genus(GENPC_DWARVEN) && one_chance_in(10))
@@ -241,12 +294,7 @@ bool transform(int pow, char which_trans)
         else
             mpr( "You turn into a living statue of rough stone." );
 
-        rem_stuff[ EQ_WEAPON ] = 0;       /* can still hold a weapon */
-        rem_stuff[ EQ_CLOAK ] = 0;       
-        rem_stuff[ EQ_HELMET ] = 0;       
-        rem_stuff[ EQ_BOOTS ] = 0;       
         // too stiff to make use of shields, gloves, or armour -- bwr
-
         remove_equipment( rem_stuff );
 
         you.attribute[ATTR_TRANSFORMATION] = TRAN_STATUE;
@@ -267,8 +315,12 @@ bool transform(int pow, char which_trans)
         return (true);
 
     case TRAN_DRAGON:  // also AC +10, ev -3, cold -1, fire +2, pois +1, flight
+        if (check_for_cursed_equipment( rem_stuff ))
+            return false;
+
         if (you.species == SP_MERFOLK && player_is_swimming())
-            mpr("You fly out of the water as you turn into a fearsome dragon!");
+            mpr("You fly out of the water as you turn into "
+                "a fearsome dragon!");
         else
             mpr("You turn into a fearsome dragon!");
 
@@ -285,12 +337,21 @@ bool transform(int pow, char which_trans)
 
         you.symbol = 'D';
         you.colour = GREEN;
+        
+        if (you.attribute[ATTR_HELD])
+        {
+            mpr("The net rips apart!");
+            you.attribute[ATTR_HELD] = 0;
+            int net = get_trapping_net(you.x_pos, you.y_pos);
+            if (net != NON_ITEM)
+                destroy_item(net);
+        }
         return (true);
 
     case TRAN_LICH:
         // also AC +3, cold +1, neg +3, pois +1, is_undead, res magic +50,
         // spec_death +1, and drain attack (if empty-handed)
-        if (you.deaths_door)
+        if (you.duration[DUR_DEATHS_DOOR])
         {
             mpr( "The transformation conflicts with an enchantment "
                  "already in effect." );
@@ -326,7 +387,14 @@ bool transform(int pow, char which_trans)
         return (true);
 
     case TRAN_AIR:
-        // also AC 20, ev +20, regen/2, no hunger, fire -2, cold -2, air +2, 
+        rem_stuff.insert(EQ_LEFT_RING);
+        rem_stuff.insert(EQ_RIGHT_RING);
+        rem_stuff.insert(EQ_AMULET);
+        
+        if (check_for_cursed_equipment( rem_stuff ))
+            return false;
+
+        // also AC 20, ev +20, regen/2, no hunger, fire -2, cold -2, air +2,
         // pois +1, spec_earth -1
         mpr( "You feel diffuse..." );
 
@@ -343,9 +411,21 @@ bool transform(int pow, char which_trans)
         modify_stat( STAT_DEXTERITY, 8, true );
         you.symbol = '#';
         you.colour = DARKGREY;
+        
+        if (you.attribute[ATTR_HELD])
+        {
+            mpr("You drift through the net!");
+            you.attribute[ATTR_HELD] = 0;
+            int net = get_trapping_net(you.x_pos, you.y_pos);
+            if (net != NON_ITEM)
+                remove_item_stationary(mitm[net]);
+        }
         return (true);
 
     case TRAN_SERPENT_OF_HELL:
+        if (check_for_cursed_equipment( rem_stuff ))
+            return false;
+
         // also AC +10, ev -5, fire +2, pois +1, life +2, slow
         mpr( "You transform into a huge demonic serpent!" );
 
@@ -363,27 +443,33 @@ bool transform(int pow, char which_trans)
         you.symbol = 'S';
         you.colour = RED;
         return (true);
+    case TRAN_NONE:
+    case NUM_TRANSFORMATIONS:
+        break;
     }
 
     return (false);
 }                               // end transform()
 
+bool transform_can_butcher_barehanded(transformation_type tt)
+{
+    return (tt == TRAN_BLADE_HANDS || tt == TRAN_DRAGON
+            || tt == TRAN_SERPENT_OF_HELL);
+}
+
 void untransform(void)
 {
-    FixedVector < char, 8 > rem_stuff;
-
-    for (int i = EQ_WEAPON; i < EQ_RIGHT_RING; i++)
-        rem_stuff[i] = 0;
-
-    you.redraw_evasion = 1;
-    you.redraw_armour_class = 1;
+    you.redraw_evasion = true;
+    you.redraw_armour_class = true;
     you.wield_change = true;
 
     you.symbol = '@';
     you.colour = LIGHTGREY;
 
     // must be unset first or else infinite loops might result -- bwr
-    const int old_form = you.attribute[ ATTR_TRANSFORMATION ];
+    const transformation_type old_form =
+        static_cast<transformation_type>(you.attribute[ ATTR_TRANSFORMATION ]);
+    
     you.attribute[ ATTR_TRANSFORMATION ] = TRAN_NONE;
     you.duration[ DUR_TRANSFORMATION ] = 0;
 
@@ -396,6 +482,12 @@ void untransform(void)
         modify_stat( STAT_DEXTERITY, -5, true );
         break;
 
+    case TRAN_BAT:
+        mpr("Your transformation has ended.", MSGCH_DURATION);
+        modify_stat( STAT_DEXTERITY, -5, true );
+        modify_stat( STAT_STRENGTH, 5, true );
+        break;
+
     case TRAN_BLADE_HANDS:
         mpr( "Your hands revert to their normal proportions.", MSGCH_DURATION );
         you.wield_change = true;
@@ -457,21 +549,24 @@ void untransform(void)
         modify_stat(STAT_STRENGTH, -13, true);
         hp_downscale = 17;
         break;
+
+    default:
+        break;
     }
 
+    if (transform_can_butcher_barehanded(old_form))
+        stop_butcher_delay();
+    
     // If nagas wear boots while transformed, they fall off again afterwards:
     // I don't believe this is currently possible, and if it is we
     // probably need something better to cover all possibilities.  -bwr
-    if ((you.species == SP_NAGA || you.species == SP_CENTAUR)
-            && you.equip[ EQ_BOOTS ] != -1
-            && you.inv[ you.equip[EQ_BOOTS] ].sub_type != ARM_NAGA_BARDING)
-    {
-        rem_stuff[EQ_BOOTS] = 1;
-        remove_equipment(rem_stuff);
-    }
 
-    calc_hp();
-    if (hp_downscale != 10)
+    // Removed barding check, no transformed creatures can wear barding
+    // anyway.
+    if (you.species == SP_NAGA || you.species == SP_CENTAUR)
+        remove_one_equip(EQ_BOOTS);
+
+    if (hp_downscale != 10 && you.hp != you.hp_max)
     {
         you.hp = you.hp * 10 / hp_downscale;
         if (you.hp < 1)
@@ -479,16 +574,17 @@ void untransform(void)
         else if (you.hp > you.hp_max)
             you.hp = you.hp_max;
     }
+    calc_hp();    
 }                               // end untransform()
 
 // XXX: This whole system is a mess as it still relies on special
 // cases to handle a large number of things (see wear_armour()) -- bwr
-bool can_equip( equipment_type use_which )
+bool can_equip( equipment_type use_which, bool ignore_temporary )
 {
 
     // if more cases are added to this if must also change in
     // item_use for naga barding
-    if (!player_is_shapechanged())
+    if (ignore_temporary || !player_is_shapechanged())
         /* or a transformation which doesn't change overall shape */
     {
         if (use_which == EQ_BOOTS)
@@ -524,27 +620,30 @@ bool can_equip( equipment_type use_which )
     if (use_which == EQ_GLOVES && you.mutation[MUT_CLAWS] >= 3)
         return (false);
 
-    switch (you.attribute[ATTR_TRANSFORMATION])
+    if (!ignore_temporary)
     {
-    case TRAN_NONE:
-    case TRAN_LICH:
-        return (true);
+        switch (you.attribute[ATTR_TRANSFORMATION])
+        {
+        case TRAN_NONE:
+        case TRAN_LICH:
+            return (true);
 
-    case TRAN_BLADE_HANDS:
-        return (use_which != EQ_WEAPON
-                && use_which != EQ_GLOVES
-                && use_which != EQ_SHIELD);
+        case TRAN_BLADE_HANDS:
+            return (use_which != EQ_WEAPON
+                    && use_which != EQ_GLOVES
+                    && use_which != EQ_SHIELD);
 
-    case TRAN_STATUE:
-        return (use_which == EQ_WEAPON 
-                || use_which == EQ_CLOAK 
-                || use_which == EQ_HELMET);
+        case TRAN_STATUE:
+            return (use_which == EQ_WEAPON 
+                    || use_which == EQ_CLOAK 
+                    || use_which == EQ_HELMET);
 
-    case TRAN_ICE_BEAST:
-        return (use_which == EQ_CLOAK);
+        case TRAN_ICE_BEAST:
+            return (use_which == EQ_CLOAK);
 
-    default:
-        return (false);
+        default:
+            return (false);
+        }
     }
 
     return (true);
